package javawrapper;

/**
 * This class packages together a set of tree settings.  For a given species
 * and tree type, this will tell you what is available in a tree map (or
 * detailed output file).
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class DetailedTreeSettings
    extends DetailedOutputSettings
    implements Cloneable {
  private int m_iType, /**<Tree type*/
      m_iSpecies; /**<Tree species*/

  /**
   * Constructor
   * @param iType Tree type.
   * @param iSpecies Tree species.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DetailedTreeSettings(int iType, int iSpecies) {
    super();
    m_iType = iType;
    m_iSpecies = iSpecies;
  }

  /**
   * Gets the tree species.
   * @return  Tree species.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetSpecies() {
    return m_iSpecies;
  }

  /**
   * Gets the tree type.
   * @return Tree type.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetType() {
    return m_iType;
  }

  /**
   * Creates a deep clone of this object.
   * @return Clone.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Object clone() {
    DetailedTreeSettings oClone = new DetailedTreeSettings(m_iType, m_iSpecies);
    DetailedOutputSettings.CopyData(oClone, this);

    return oClone;
  }
}
