package javawrapper;

/**
* Manages substrate data and behaviors.
* <p>Copyright: Copyright (c) 2003 Charles D. Canham</p>
* <p>Company: Institute of Ecosystem Studies</p>
* @author Lora E. Murphy
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)
* <br>July 21, 2004:  Added calcs grids (LEM)
* <br>April 1, 2005:  Changed some substrate names per CDC (LEM)
* <br>April 22, 2005:  Added new snag uproot parameter (LEM)
* <br>June 3, 2005:  Added combined substrate (LEM)
*/

public class SubstrateBehaviors
    extends BehaviorTypeBase {

  protected ModelVector
      /**Proportion of dead that fall for each species.*/
      mp_fProportionOfDeadThatFall = new ModelVector(
      "Proportion of Dead that Fall", "su_propOfDeadFall", "su_podfVal", 0,
      ModelVector.FLOAT);

  /**Proportion of fallen that uproot for each species.*/
  protected ModelVector mp_fProportionOfFallenThatUproot = new ModelVector(
      "Proportion of Fallen that Uproot", "su_propOfFallUproot", "su_pofuVal",
      0, ModelVector.FLOAT);

  /**Proportion of snags that uproot for each species. LEM 04/22/05*/
  protected ModelVector mp_fProportionOfSnagsThatUproot = new ModelVector(
      "Proportion of Snags that Uproot", "su_propOfSnagsUproot", "su_posuVal",
      0, ModelVector.FLOAT);

  protected ModelFloat
      /**Scarified soil decay alpha*/
      m_fScarSoilA = new ModelFloat(0, "Scarified Soil Annual Decay Alpha",
                                   "su_scarSoilDecayAlpha");

  /**Scarified soil decay beta*/
  protected ModelFloat m_fScarSoilB = new ModelFloat(0,
      "Scarified Soil Annual Decay Beta",
      "su_scarSoilDecayBeta");

  /**Tip-Up Mounds decay alpha*/
  protected ModelFloat m_fTipUpA = new ModelFloat(0,
      "Tip-Up Mounds Annual Decay Alpha", "su_tipupDecayAlpha");

  /**Tip-Up Mounds decay beta*/
  protected ModelFloat m_fTipUpB = new ModelFloat(0,
      "Tip-Up Mounds Annual Decay Beta", "su_tipupDecayBeta");

  /**Fresh log decay alpha*/
  protected ModelFloat m_fFreshlogA = new ModelFloat(0,
      "Fresh Log Annual Decay Alpha", "su_freshLogDecayAlpha");

  /**Fresh log decay beta*/
  protected ModelFloat m_fFreshlogB = new ModelFloat(0,
      "Fresh Log Annual Decay Beta", "su_freshLogDecayBeta");

  /**Decayed log to forest floor decay term*/
  protected ModelFloat m_fDecLogToForestFloor = new ModelFloat(0,
      "Decayed Log to Forest Floor Annual Decay", "su_DecayedLogToForestFloor");

  /**Scarified soil initial condition proportion*/
  protected ModelFloat m_fInitCondScarSoil = new ModelFloat(0,
      "Initial Conditions Proportion of Scarified Soil", "su_initialScarSoil");

  /**Scarified soil partial cut proportion*/
  protected ModelFloat m_fPartialCutScarSoil = new ModelFloat(0,
      "Partial Cut Proportion of Scarified Soil",
      "su_partialCutScarSoil");

  /**Scarified soil gap cut proportion*/
  protected ModelFloat m_fGapCutScarSoil = new ModelFloat(0,
      "Gap Cut Proportion of Scarified Soil",
      "su_gapCutScarSoil");

  /**Scarified soil clear cut proportion*/
  protected ModelFloat m_fClearCutScarSoil = new ModelFloat(0,
      "Clear Cut Proportion of Scarified Soil",
      "su_clearCutScarSoil");

  /**Tip-Up Mounds initial condition proportion*/
  protected ModelFloat m_fInitCondTipup = new ModelFloat(0,
      "Initial Conditions Proportion of Tip-Up Mounds", "su_initialTipup");

  /**Tip-Up Mounds partial cut proportion*/
  protected ModelFloat m_fPartialCutTipup = new ModelFloat(0,
      "Partial Cut Proportion of Tip-Up Mounds",
      "su_partialCutTipup");

  /**Tip-Up Mounds gap cut proportion*/
  protected ModelFloat m_fGapCutTipup = new ModelFloat(0,
      "Gap Cut Proportion of Tip-Up Mounds",
      "su_gapCutTipup");

  /**Tip-Up Mounds clear cut proportion*/
  protected ModelFloat m_fClearCutTipup = new ModelFloat(0,
      "Clear Cut Proportion of Tip-Up Mounds",
      "su_clearCutTipup");

  /**Fresh log initial condition proportion*/
  protected ModelFloat m_fInitCondFreshLog = new ModelFloat(0,
      "Initial Conditions Proportion of Fresh Logs", "su_initialFreshLog");

  /**Fresh log partial cut proportion*/
  protected ModelFloat m_fPartialCutFreshLog = new ModelFloat(0,
      "Partial Cut Proportion of Fresh Logs", "su_partialCutFreshLog");

  /**Fresh log gap cut proportion*/
  protected ModelFloat m_fGapCutFreshLog = new ModelFloat(0,
      "Gap Cut Proportion of Fresh Logs",
      "su_gapCutFreshLog");

  /**Fresh log clear cut proportion*/
  protected ModelFloat m_fClearCutFreshLog = new ModelFloat(0,
      "Clear Cut Proportion of Fresh Logs",
      "su_clearCutFreshLog");

  /**Decayed log initial condition proportion*/
  protected ModelFloat m_fInitCondDecLog = new ModelFloat(0,
      "Initial Conditions Proportion of Decayed Logs", "su_initialDecayedLog");

  /**Decayed log partial cut proportion*/
  protected ModelFloat m_fPartialCutDecLog = new ModelFloat(0,
      "Partial Cut Proportion of Decayed Logs", "su_partialCutDecayedLog");

  /**Decayed log gap cut proportion*/
  protected ModelFloat m_fGapCutDecLog = new ModelFloat(0,
      "Gap Cut Proportion of Decayed Logs",
      "su_gapCutDecayedLog");

  /**Decayed log clear cut proportion*/
  protected ModelFloat m_fClearCutDecLog = new ModelFloat(0,
      "Clear Cut Proportion of Decayed Logs",
      "su_clearCutDecayedLog");

  /**Root soil disturbance factor*/
  protected ModelFloat m_fRootTipupFactor = new ModelFloat(0,
      "Uprooted Tree Radius Increase Factor for Root Rip-Out",
      "su_rootTipupFactor");

  /**Proportion of litter/moss that is moss*/
  protected ModelFloat m_fMossProportion = new ModelFloat(0,
      "Proportion of Forest Floor Litter/Moss Pool that is Moss", "su_mossProportion");

  /**Whether or not tree fall is directional - LEM 06/03/05*/
  protected ModelEnum m_iDirectionalTreeFall =
      new ModelEnum(new int[] {0, 1},
                    new String[] {"false", "true"},
                    "Use Directional Tree Fall",
                    "su_directionalTreeFall");

  protected ModelInt
      /**Maximum number of years a substrate event hangs around*/
      m_iMaxDecayTime = new ModelInt(0,
                                     "Maximum Number of Years that Decay Occurs",
                                     "su_maxNumberDecayYears");

  /**
   * Constructor
   * @param oManager GUIManager object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public SubstrateBehaviors(GUIManager oManager) {
    super(oManager, "Substrate", "substrate");

    m_sHelpID = "data.substrate_behaviors";

    //Set up our child behavior vector
    mp_oChildBehaviors = new Behavior[1];
    mp_oChildBehaviors[0] = new Behavior("substrate", "Substrate", "substrate",
                                         (float) 2);
      //Substrate cannot apply to seedlings
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEEDLING, false);

    mp_oChildBehaviors[0].AddRequiredData(m_iMaxDecayTime);
    mp_oChildBehaviors[0].AddRequiredData(m_fScarSoilA);
    mp_oChildBehaviors[0].AddRequiredData(m_fScarSoilB);
    mp_oChildBehaviors[0].AddRequiredData(m_fTipUpA);
    mp_oChildBehaviors[0].AddRequiredData(m_fTipUpB);
    mp_oChildBehaviors[0].AddRequiredData(m_fFreshlogA);
    mp_oChildBehaviors[0].AddRequiredData(m_fFreshlogB);
    mp_oChildBehaviors[0].AddRequiredData(m_fDecLogToForestFloor);
    mp_oChildBehaviors[0].AddRequiredData(m_iDirectionalTreeFall);
    mp_oChildBehaviors[0].AddRequiredData(m_fMossProportion);
    mp_oChildBehaviors[0].AddRequiredData(m_fRootTipupFactor);
    mp_oChildBehaviors[0].AddRequiredData(m_fInitCondScarSoil);
    mp_oChildBehaviors[0].AddRequiredData(m_fInitCondTipup);
    mp_oChildBehaviors[0].AddRequiredData(m_fInitCondFreshLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fInitCondDecLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fPartialCutScarSoil);
    mp_oChildBehaviors[0].AddRequiredData(m_fPartialCutTipup);
    mp_oChildBehaviors[0].AddRequiredData(m_fPartialCutFreshLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fPartialCutDecLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fGapCutScarSoil);
    mp_oChildBehaviors[0].AddRequiredData(m_fGapCutTipup);
    mp_oChildBehaviors[0].AddRequiredData(m_fGapCutFreshLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fGapCutDecLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearCutScarSoil);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearCutTipup);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearCutFreshLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearCutDecLog);
    mp_oChildBehaviors[0].AddRequiredData(mp_fProportionOfDeadThatFall);
    mp_oChildBehaviors[0].AddRequiredData(mp_fProportionOfFallenThatUproot);
    mp_oChildBehaviors[0].AddRequiredData(mp_fProportionOfSnagsThatUproot);

    mp_oAllData.add(m_fScarSoilA);
    mp_oAllData.add(m_fScarSoilB);
    mp_oAllData.add(m_fTipUpA);
    mp_oAllData.add(m_fTipUpB);
    mp_oAllData.add(m_fFreshlogA);
    mp_oAllData.add(m_fFreshlogB);
    mp_oAllData.add(m_fDecLogToForestFloor);
    mp_oAllData.add(m_iMaxDecayTime);
    mp_oAllData.add(m_fInitCondScarSoil);
    mp_oAllData.add(m_fInitCondTipup);
    mp_oAllData.add(m_fInitCondFreshLog);
    mp_oAllData.add(m_fInitCondDecLog);
    mp_oAllData.add(m_fPartialCutScarSoil);
    mp_oAllData.add(m_fPartialCutTipup);
    mp_oAllData.add(m_fPartialCutFreshLog);
    mp_oAllData.add(m_fPartialCutDecLog);
    mp_oAllData.add(m_fGapCutScarSoil);
    mp_oAllData.add(m_fGapCutTipup);
    mp_oAllData.add(m_fGapCutFreshLog);
    mp_oAllData.add(m_fGapCutDecLog);
    mp_oAllData.add(m_fClearCutScarSoil);
    mp_oAllData.add(m_fClearCutTipup);
    mp_oAllData.add(m_fClearCutFreshLog);
    mp_oAllData.add(m_fClearCutDecLog);
    mp_oAllData.add(mp_fProportionOfDeadThatFall);
    mp_oAllData.add(mp_fProportionOfFallenThatUproot);
    mp_oAllData.add(mp_fProportionOfSnagsThatUproot);
    mp_oAllData.add(m_fRootTipupFactor);
    mp_oAllData.add(m_fMossProportion);
    mp_oAllData.add(m_iDirectionalTreeFall);
  }

  /**
   * Sets up the substrate grids.
   * @param oPop Tree population.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>July 21, 2004:  Added calcs grids (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {
    mp_oGrids = new Grid[2];

    //************************
     // Regular substrate
     //************************
    String sGridName = "Substrate";

    //Create the six data members
    DataMember[] p_oDataMembers = new DataMember[6];

    p_oDataMembers[0] = new DataMember("Proportion of scarified soil",
                                       "scarsoil",
                                       DataMember.FLOAT);
    p_oDataMembers[1] = new DataMember("Proportion of tip-up mounds",
                                       "tipup", DataMember.FLOAT);
    p_oDataMembers[2] = new DataMember("Proportion of fresh logs", "freshlog",
                                       DataMember.FLOAT);
    p_oDataMembers[3] = new DataMember("Proportion of decayed logs", "declog",
                                       DataMember.FLOAT);
    p_oDataMembers[4] = new DataMember("Proportion of forest floor moss", "ffmoss",
                                       DataMember.FLOAT);
    p_oDataMembers[5] = new DataMember("Proportion of forest floor litter", "fflitter",
                                       DataMember.FLOAT);

    DataMember[] p_oPackageDataMembers = new DataMember[4];

    p_oPackageDataMembers[0] = new DataMember("Substrate cohort age", "age",
                                              DataMember.INTEGER);
    p_oPackageDataMembers[1] = new DataMember(
        "Substrate cohort new scarified soil subtrate", "scarsoil",
        DataMember.FLOAT);
    p_oPackageDataMembers[2] = new DataMember(
        "Substrate cohort new tip-up mounds substrate",
        "tipup", DataMember.FLOAT);
    p_oPackageDataMembers[3] = new DataMember("Substrate cohort new fresh logs",
                                              "freshlog",
                                              DataMember.FLOAT);

    mp_oGrids[0] = new Grid(sGridName, p_oDataMembers, p_oPackageDataMembers, 8,
                            8);

    //Add to the substrate behavior
    GetBehaviorByKey("substrate").AddGrid(mp_oGrids[0]);

    //************************
     // Regular substrate calculations grid
     //************************
    sGridName = "substratecalcs";

    //Create the data members
    int iDecayTimesteps = (int) java.lang.Math.ceil( (float) m_iMaxDecayTime.
        GetValue() / m_oManager.GetPlot().GetNumberOfYearsPerTimestep());
    p_oDataMembers = new DataMember[1 + iDecayTimesteps];

    p_oDataMembers[0] = new DataMember("Amount of new tip-up mounds",
                                       "newtipup",
                                       DataMember.FLOAT);
    for (int i = 0; i < iDecayTimesteps; i++) {
      p_oDataMembers[1 +
          i] = new DataMember("Fresh logs " + String.valueOf(i) + " timesteps ago",
                              "freshlog_" + String.valueOf(i),
                              DataMember.FLOAT);
    }
    mp_oGrids[1] = new Grid(sGridName, p_oDataMembers, null, 8,
                            8);
    //Add to the substrate behavior
    GetBehaviorByKey("substrate").AddGrid(mp_oGrids[1]);

    //************************
     // Defaults
     //************************
      //Set the proportion of snags that uproot to all 0s, in case snags aren't
      //being used
    int iNumSpecies = oPop.GetNumberOfSpecies(), i;
    if (mp_fProportionOfSnagsThatUproot.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fProportionOfSnagsThatUproot.GetValue().add(new Float(0));
      }
    }

  }

  /**
   * Sets the scarified soil decay alpha parameter.
   * @param f Scarified soil decay alpha.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGapMossDecayAlpha(float f) {
    m_fScarSoilA.SetValue(f);
  }

  /**
   * Sets the scarified soil decay beta parameter.
   * @param f Scarified soil decay beta.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGapMossDecayBeta(float f) {
    m_fScarSoilB.SetValue(f);
  }

  /**
   * Sets the tip-up mounds decay alpha parameter.
   * @param f Tip-Up Mounds decay alpha.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetOrganicsDecayAlpha(float f) {
    m_fTipUpA.SetValue(f);
  }

  /**
   * Sets the tip-up mounds decay beta parameter.
   * @param f Tip-Up Mounds decay beta.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetOrganicsDecayBeta(float f) {
    m_fTipUpB.SetValue(f);
  }

  /**
   * Sets the fresh log decay alpha parameter.
   * @param f Fresh log decay alpha.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetFreshLogDecayAlpha(float f) {
    m_fFreshlogA.SetValue(f);
  }

  /**
   * Sets the fresh log decay beta parameter.
   * @param f Fresh log decay beta.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetFreshLogDecayBeta(float f) {
    m_fFreshlogB.SetValue(f);
  }

  /**
   * Sets the mossy log to forest floor decay parameter.
   * @param f Mossy log to forest floor decay parameter.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetMossyLogToCanopyMoss(float f) {
    m_fDecLogToForestFloor.SetValue(f);
  }

  /**
   * Sets the maximum amount of time a substrate event is allowed to decay.
   * This triggers DoSetup, because the grids need to be redone.
   * @param i Maximum decay time, in timesteps.
   * @throws ModelException if the decay time is less than zero.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetMaximumDecayTime(int i) throws ModelException {
    ModelInt iTest = new ModelInt(i, m_iMaxDecayTime.GetDescriptor(), "");
    MakeSurePositive(iTest);
    m_iMaxDecayTime.SetValue(i);
    DoSetup(m_oManager.GetTreePopulation());
  }

  /**
   * Overridden from WorkerBase.  This is on the lookout for changes to
   * m_iMaxDecayTime, so DoSetup() can be triggered and the grids recreated.
   * @param oDataMember Data member to set.
   * @param oData Data value to set into data member.
   * @throws ModelException if data is of the wrong type.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected void SetSingleValue(ModelData oDataMember, Object oData) throws
      ModelException {
    super.SetSingleValue(oDataMember, oData);
    if (oDataMember.equals(m_iMaxDecayTime)) {
      DoSetup(m_oManager.GetTreePopulation());
    }
  }

  /**
   * Sets the proportion of fresh logs after a clear cut.
   * @param f Proportion of fresh logs, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetClearCutFreshLog(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fClearCutFreshLog.GetDescriptor(),
                                      "");
    MakeSureIsProportion(fTest);
    m_fClearCutFreshLog.SetValue(f);
  }

  /**
   * Sets the proportion of scarified soil after a clear cut.
   * @param f Proportion of scarified soil, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetClearCutGapMoss(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fClearCutScarSoil.GetDescriptor(), "");
    MakeSureIsProportion(fTest);
    m_fClearCutScarSoil.SetValue(f);
  }

  /**
   * Sets the proportion of decayed logs after a clear cut.
   * @param f Proportion of decayed logs, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetClearCutMossyLog(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fClearCutDecLog.GetDescriptor(),
                                      "");
    MakeSureIsProportion(fTest);
    m_fClearCutDecLog.SetValue(f);
  }

  /**
   * Sets the proportion of tip-up mounds after a clear cut.
   * @param f Proportion of tip-up mounds, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetClearCutOrganics(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fClearCutTipup.GetDescriptor(), "");
    MakeSureIsProportion(fTest);
    m_fClearCutTipup.SetValue(f);
  }

  /**
   * Sets the proportion of fresh logs after a partial cut.
   * @param f Proportion of fresh logs, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetPartialCutFreshLog(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fPartialCutFreshLog.GetDescriptor(),
                                      "");
    MakeSureIsProportion(fTest);
    m_fPartialCutFreshLog.SetValue(f);
  }

  /**
   * Sets the proportion of scarified soil after a partial cut.
   * @param f Proportion of scarified soil, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetPartialCutGapMoss(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fPartialCutScarSoil.GetDescriptor(),
                                      "");
    MakeSureIsProportion(fTest);
    m_fPartialCutScarSoil.SetValue(f);
  }

  /**
   * Sets the proportion of decayed logs after a partial cut.
   * @param f Proportion of decayed logs, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetPartialCutMossyLog(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fPartialCutDecLog.GetDescriptor(),
                                      "");
    MakeSureIsProportion(fTest);
    m_fPartialCutDecLog.SetValue(f);
  }

  /**
   * Sets the proportion of tip-up mounds after a partial cut.
   * @param f Proportion of tip-up mounds, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetPartialCutOrganics(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fPartialCutTipup.GetDescriptor(),
                                      "");
    MakeSureIsProportion(fTest);
    m_fPartialCutTipup.SetValue(f);
  }

  /**
   * Sets the proportion of fresh logs as initial conditions.
   * @param f Proportion of fresh logs, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetInitConditionsFreshLog(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fInitCondFreshLog.GetDescriptor(),
                                      "");
    MakeSureIsProportion(fTest);
    m_fInitCondFreshLog.SetValue(f);
  }

  /**
   * Sets the proportion of scarified soil as initial conditions.
   * @param f Proportion of scarified soil, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetInitConditionsGapMoss(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fInitCondScarSoil.GetDescriptor(), "");
    MakeSureIsProportion(fTest);
    m_fInitCondScarSoil.SetValue(f);
  }

  /**
   * Sets the proportion of decayed logs as initial conditions.
   * @param f Proportion of decayed logs, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetInitConditionsMossyLog(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fInitCondDecLog.GetDescriptor(),
                                      "");
    MakeSureIsProportion(fTest);
    m_fInitCondDecLog.SetValue(f);
  }

  /**
   * Sets the proportion of tip-up mounds as initial conditions.
   * @param f Proportion of tip-up mounds, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetInitConditionsOrganics(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fInitCondTipup.GetDescriptor(), "");
    MakeSureIsProportion(fTest);
    m_fInitCondTipup.SetValue(f);
  }

  /**
   * Sets the proportion of fresh logs after a gap cut.
   * @param f Proportion of fresh logs, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGapCutFreshLog(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fGapCutFreshLog.GetDescriptor(), "");
    MakeSureIsProportion(fTest);
    m_fGapCutFreshLog.SetValue(f);
  }

  /**
   * Sets the proportion of scarified soil after a gap cut.
   * @param f Proportion of scarified soil, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGapCutGapMoss(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fGapCutScarSoil.GetDescriptor(), "");
    MakeSureIsProportion(fTest);
    m_fGapCutScarSoil.SetValue(f);
  }

  /**
   * Sets the proportion of decayed logs after a gap cut.
   * @param f Proportion of decayed logs, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGapCutMossyLog(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fGapCutDecLog.GetDescriptor(), "");
    MakeSureIsProportion(fTest);
    m_fGapCutDecLog.SetValue(f);
  }

  /**
   * Sets the proportion of tip-up mounds after a gap cut.
   * @param f Proportion of tip-up mounds, between 0 and 1.
   * @throws ModelException if the value passed is not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGapCutOrganics(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fGapCutTipup.GetDescriptor(), "");
    MakeSureIsProportion(fTest);
    m_fGapCutTipup.SetValue(f);
  }

  /**
   * Sets the proportion of dead trees that fall over for all species.
   * @param p_fVals Proportions for each species in order of species number.
   * @throws ModelException if the values are not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetProportionOfDeadThatFall(Float[] p_fVals) throws
      ModelException {
    ModelVector oData = new ModelVector(mp_fProportionOfDeadThatFall.
                                        GetDescriptor(), "", "", 0,
                                        ModelVector.FLOAT);
    SetVectorValues(oData, p_fVals);
    MakeSureAllAreProportions(oData);
    SetVectorValues(mp_fProportionOfDeadThatFall, p_fVals);
  }

  /**
   * Sets the proportion of fallen trees that uproot for all species.
   * @param p_fVals Proportions for each species in order of species number.
   * @throws ModelException if the values are not between 0 and 1.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetProportionOfFallenThatUproot(Float[] p_fVals) throws
      ModelException {
    ModelVector oData = new ModelVector(mp_fProportionOfFallenThatUproot.
                                        GetDescriptor(), "", "", 0,
                                        ModelVector.FLOAT);
    SetVectorValues(oData, p_fVals);
    MakeSureAllAreProportions(oData);
    SetVectorValues(mp_fProportionOfFallenThatUproot, p_fVals);
  }

  /**
   * Sets the value of the tip-up mound area.
   * @param f Tip-up mound area, in square meters.
   * @throws ModelException if the value passed is less than 0.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetRootFactor(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fRootTipupFactor.GetDescriptor(), "");
    MakeSurePositive(fTest);
    m_fRootTipupFactor.SetValue(f);
  }

  /**
   * Sets whether or not the substrate behavior is to be used.
   * <p>
   * This method would normally not be used, as this object manages its own
   * data and would make the decision of which behaviors are enabled and which
   * are not.  However, in the case of input of a tab-delimited parameter file,
   * the information must be fed to this class.
   * @param bUse Whether or not substrate is to be used.
   * @param oPop TreePopulation object.
   * @throws ModelException if something goes wrong with species/type setup.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetUseSubstrate(boolean bUse, TreePopulation oPop) throws
      ModelException {

    mp_oChildBehaviors[0].m_bIsEnabled = bUse;

    //Apply this to all saplings and adults if it's supposed to be used
    if (true == bUse) {
      Integer[] p_iFlags = new Integer[oPop.GetNumberOfSpecies()];
      int i;
      for (i = 0; i < oPop.GetNumberOfSpecies(); i++) {
        p_iFlags[i] = new Integer(1);
      }
      SetUsesBehavior(p_iFlags, TreePopulation.SAPLING, oPop, "substrate");
      SetUsesBehavior(p_iFlags, TreePopulation.ADULT, oPop, "substrate");
    }
  }

  /**
   * Overridden to provide backwards compatibility - I changed XML tag
   * names.  This was put in for 6.05 - remove after several more versions.
   * @param sXMLTag XML tag of data object whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param oAttributes Attributes of the object.  Ignored, but may be needed
   * by overriding objects.
   * @param oData Data value, either String or type appropriate to the data
   * type
   * @return true if the value was set successfully; false if the value could
   * not be found.
   * @throws ModelException if the value could not be assigned to the data
   * object.
   */
  public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        org.xml.sax.Attributes oAttributes,
                                        Object oData) throws
      ModelException {

    if (sXMLParentTag.equals(m_sXMLTag)) {
      if (sXMLTag.equals("su_gapmossDecayAlpha")) {
        sXMLTag = "su_scarSoilDecayAlpha";
      }
      else if (sXMLTag.equals("su_gapmossDecayBeta")) {
        sXMLTag = "su_scarSoilDecayBeta";
      }
      else if (sXMLTag.equals("su_organicDecayAlpha")) {
        sXMLTag = "su_tipupDecayAlpha";
      }
      else if (sXMLTag.equals("su_organicDecayBeta")) {
        sXMLTag = "su_tipupDecayBeta";
      }
      else if (sXMLTag.equals("su_mossyLogToCanMoss")) {
        sXMLTag = "su_DecayedLogToForestFloor";
      }
      else if (sXMLTag.equals("su_initialGapMoss")) {
        sXMLTag = "su_initialScarSoil";
      }
      else if (sXMLTag.equals("su_initialOrganic")) {
        sXMLTag = "su_initialTipup";
      }
      else if (sXMLTag.equals("su_initialMossyLog")) {
        sXMLTag = "su_initialDecayedLog";
      }
      else if (sXMLTag.equals("su_partialCutGapMoss")) {
        sXMLTag = "su_partialCutScarSoil";
      }
      else if (sXMLTag.equals("su_partialCutOrganic")) {
        sXMLTag = "su_partialCutTipup";
      }
      else if (sXMLTag.equals("su_partialCutMossyLog")) {
        sXMLTag = "su_partialCutDecayedLog";
      }
      else if (sXMLTag.equals("su_gapCutGapMoss")) {
        sXMLTag = "su_gapCutScarSoil";
      }
      else if (sXMLTag.equals("su_gapCutOrganic")) {
        sXMLTag = "su_gapCutTipup";
      }
      else if (sXMLTag.equals("su_gapCutMossyLog")) {
        sXMLTag = "su_gapCutDecayedLog";
      }
      else if (sXMLTag.equals("su_clearCutGapMoss")) {
        sXMLTag = "su_clearCutScarSoil";
      }
      else if (sXMLTag.equals("su_clearCutOrganic")) {
        sXMLTag = "su_clearCutTipup";
      }
      else if (sXMLTag.equals("su_clearCutMossyLog")) {
        sXMLTag = "su_clearCutDecayedLog";
      }
    }
    return super.SetSingleValueByXMLTag(sXMLTag, sXMLParentTag, oAttributes,
                                            oData);
  }

  /**
   * Validates the data in preparation for parameter file writing or some such.
   * @throws ModelException if:
   * <ul>
   * <li>m_fRootTipupFactor or m_iMaxDecayTime are not positive numbers</li>
   * <li>if the values in mp_fProportionOfDeadThatFall,
   * mp_fProportionOfFallenThatUproot, and mp_fProportionOfSnagsThatUproot are
   * not proportions</li>
   * <li>m_fMossProportion isn't a proportion</li>
   * <li>any initial or harvest conditions value isn't a proportion</li>
   * <li>the total of any set of initial or harvest conditions is greater than
   * 1</li>
   * </ul>
   * @param oPop Not used.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>April 22, 2005:  Added proportion checking (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {

    if (GetBehaviorByKey("substrate").m_bIsEnabled) {
      boolean[] p_bApplies = GetBehaviorByKey("substrate").GetWhichSpeciesUsed(oPop);

      MakeSurePositive(m_fRootTipupFactor);
      MakeSurePositive(m_iMaxDecayTime);
      MakeSureIsProportion(m_fMossProportion);
      MakeSureAllAreProportions(mp_fProportionOfDeadThatFall, p_bApplies);
      MakeSureAllAreProportions(mp_fProportionOfFallenThatUproot, p_bApplies);
      MakeSureAllAreProportions(mp_fProportionOfSnagsThatUproot, p_bApplies);
      MakeSureIsProportion(m_fInitCondDecLog);
      MakeSureIsProportion(m_fInitCondFreshLog);
      MakeSureIsProportion(m_fInitCondScarSoil);
      MakeSureIsProportion(m_fInitCondTipup);
      MakeSureIsProportion(m_fPartialCutDecLog);
      MakeSureIsProportion(m_fPartialCutFreshLog);
      MakeSureIsProportion(m_fPartialCutScarSoil);
      MakeSureIsProportion(m_fPartialCutTipup);
      MakeSureIsProportion(m_fGapCutDecLog);
      MakeSureIsProportion(m_fGapCutFreshLog);
      MakeSureIsProportion(m_fGapCutScarSoil);
      MakeSureIsProportion(m_fGapCutTipup);
      MakeSureIsProportion(m_fClearCutDecLog);
      MakeSureIsProportion(m_fClearCutFreshLog);
      MakeSureIsProportion(m_fClearCutScarSoil);
      MakeSureIsProportion(m_fClearCutTipup);

      if (m_fInitCondDecLog.GetValue() +
          m_fInitCondFreshLog.GetValue() +
          m_fInitCondScarSoil.GetValue() +
          m_fInitCondTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The initial conditions values must not add up to be greater than 1.");
      }

      if (m_fPartialCutDecLog.GetValue() +
          m_fPartialCutFreshLog.GetValue() +
          m_fPartialCutScarSoil.GetValue() +
          m_fPartialCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The partial cut values must not add up to be greater than 1.");
      }

      if (m_fGapCutDecLog.GetValue() +
          m_fGapCutFreshLog.GetValue() +
          m_fGapCutScarSoil.GetValue() +
          m_fGapCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The gap cut values must not add up to be greater than 1.");
      }

      if (m_fClearCutDecLog.GetValue() +
          m_fClearCutFreshLog.GetValue() +
          m_fClearCutScarSoil.GetValue() +
          m_fClearCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The clear cut values must not add up to be greater than 1.");
      }
    }
  }
}
