package javawrapper;

import junit.framework.*;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.InputSource;
import java.io.FileWriter;
import java.io.File;
import java.util.Vector;

/**
 * Tests the AnalysisBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestAnalysisBehaviors
    extends TestCase {

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Tests reading of parameter file XML volume calculator settings.
   */
  public void testReadXMLVolumeSettings() {
    GUIManager oManager = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteVolumeXMLFile1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( ( (Float) oAnalysis.mp_fTaperA.GetValue().elementAt(1)).
                   floatValue(), 0.843142, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperA.GetValue().elementAt(2)).
                   floatValue(), 0.8, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperB.GetValue().elementAt(1)).
                   floatValue(), 1.00813, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperB.GetValue().elementAt(2)).
                   floatValue(), 0.9, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperC.GetValue().elementAt(1)).
                   floatValue(), 0.031402, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperC.GetValue().elementAt(2)).
                   floatValue(), 0.032, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperD.GetValue().elementAt(1)).
                   floatValue(), 0.319773, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperD.GetValue().elementAt(2)).
                   floatValue(), 0.321, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperF.GetValue().elementAt(1)).
                   floatValue(), -0.532732, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperF.GetValue().elementAt(2)).
                   floatValue(), -0.51, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperG.GetValue().elementAt(1)).
                   floatValue(), 0.585361, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperG.GetValue().elementAt(2)).
                   floatValue(), 0.59, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperI.GetValue().elementAt(1)).
                   floatValue(), 1.0315, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperI.GetValue().elementAt(2)).
                   floatValue(), 1.04, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperJ.GetValue().elementAt(1)).
                   floatValue(), 0.018622, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperJ.GetValue().elementAt(2)).
                   floatValue(), 0.02, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperK.GetValue().elementAt(1)).
                   floatValue(), -0.225146, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperK.GetValue().elementAt(2)).
                   floatValue(), -0.23, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkA.GetValue().elementAt(1)).
                   floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkA.GetValue().elementAt(2)).
                   floatValue(), 0.01, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkB.GetValue().elementAt(1)).
                   floatValue(), 0.8, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkB.GetValue().elementAt(2)).
                   floatValue(), 1, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkC.GetValue().elementAt(1)).
                   floatValue(), 0.01, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkC.GetValue().elementAt(2)).
                   floatValue(), 0.001, 0.001);

      assertEquals(oAnalysis.m_fMinUsableDiam.GetValue(), 3, 0.001);
      assertEquals(oAnalysis.m_fSegmentLength.GetValue(), 2, 0.001);
      assertEquals(oAnalysis.m_fStumpHeight.GetValue(), 13, 0.001);

      new File(sFileName).delete();
      System.out.println("XML volume reading test succeeded.");
    }
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.sMessage);
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests reading of parameter file XML biomass calculator settings.
   */
  public void testReadXMLBiomassSettings() {
    GUIManager oManager = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteBiomassXMLFile1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(0)).
                   floatValue(), 0.1, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(1)).
                   floatValue(), 2.4, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(2)).
                   floatValue(), 10, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(0)).
                   floatValue(), -0.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(1)).
                   floatValue(), 0.03, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(2)).
                   floatValue(), 2, 0.001);

      new File(sFileName).delete();
      System.out.println("XML biomass reading test succeeded.");
    }
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.sMessage);
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  public void testFormatDataForDisplay() {
    try {
      TreePopulation oPop = new TreePopulation(null);
      oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});

      AnalysisBehaviors oAnalysis = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no Analysis behaviors are enabled.
      oAnalysis = new AnalysisBehaviors(null);
      assertEquals(null, oAnalysis.FormatDataForDisplay(oPop));

      //Case:  only tree volume is enabled.
      oAnalysis = null;
      oAnalysis = new AnalysisBehaviors(null);
      oAnalysis.DoSetup(oPop);
      oBeh = oAnalysis.GetBehaviorByKey("tree volume calculator");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Taper Equation Initial Multiplier (a0)",
          "Taper Equation DBH Exponent (a1)",
          "Taper Equation Height Exponent (a2)",
          "Taper Equation X Exponent 1 (b1)",
          "Taper Equation X Exponent 2 (b2)",
          "Taper Equation X Exponent 3 (b3)",
          "Taper Equation X Exponent 4 (b4)",
          "Taper Equation X Exponent 5 (b5)",
          "Taper Equation X Exponent 6 (b6)",
          "Diameter-Outside-Bark Constant (a1)",
          "Diameter-Outside-Bark First Degree Parameter (a2)",
          "Diameter-Outside-Bark Second Degree Parameter (a3)",
          "Height to Begin Calculating Trunk Volume, in cm",
          "Minimum Trunk Diameter for Volume Calculations, in cm",
          "Trunk Segment Length for Volume Calculations, in m"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only tree biomass is enabled.
      oAnalysis = null;
      oAnalysis = new AnalysisBehaviors(null);
      oAnalysis.DoSetup(oPop);
      oBeh = oAnalysis.GetBehaviorByKey("tree biomass calculator");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Dimension Analysis Equation ID",
          "Dimension Analysis Biomass Units",
          "Dimension Analysis: Use Correction Factor?",
          "Dimension Analysis Correction Factor",
          "Dimension Analysis Parameter (a)",
          "Dimension Analysis Parameter (b)",
          "Dimension Analysis Parameter (c)",
          "Dimension Analysis Parameter (d)",
          "Dimension Analysis Parameter (e)",
          "Dimension Analysis DBH Units"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only tree biomass is enabled.
      oAnalysis = null;
      oAnalysis = new AnalysisBehaviors(null);
      oAnalysis.DoSetup(oPop);
      oBeh = oAnalysis.GetBehaviorByKey("bole volume");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Bole Volume Parameter (b0)",
          "Bole Volume Parameter (b1)",
          "Bole Volume Parameter (b2)",
          "Bole Volume Parameter (b3)",
          "Bole Volume Parameter (b4)",
          "Bole Volume Parameter (b5)",
          "Bole Volume Form Class, 60% - 100%"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  all are enabled.
      oAnalysis = null;
      oAnalysis = new AnalysisBehaviors(null);
      oAnalysis.DoSetup(oPop);
      int i;
      for (i = 0; i < oAnalysis.mp_oChildBehaviors.length; i++) {
        oAnalysis.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oAnalysis.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
            SpeciesTypeCombo(0, 3, oPop));
      }
      p_sExpected = new String[] {
          "Taper Equation Initial Multiplier (a0)",
          "Taper Equation DBH Exponent (a1)",
          "Taper Equation Height Exponent (a2)",
          "Taper Equation X Exponent 1 (b1)",
          "Taper Equation X Exponent 2 (b2)",
          "Taper Equation X Exponent 3 (b3)",
          "Taper Equation X Exponent 4 (b4)",
          "Taper Equation X Exponent 5 (b5)",
          "Taper Equation X Exponent 6 (b6)",
          "Diameter-Outside-Bark Constant (a1)",
          "Diameter-Outside-Bark First Degree Parameter (a2)",
          "Diameter-Outside-Bark Second Degree Parameter (a3)",
          "Height to Begin Calculating Trunk Volume, in cm",
          "Minimum Trunk Diameter for Volume Calculations, in cm",
          "Trunk Segment Length for Volume Calculations, in m",
          "Dimension Analysis Equation ID",
          "Dimension Analysis Biomass Units",
          "Dimension Analysis: Use Correction Factor?",
          "Dimension Analysis Correction Factor",
          "Dimension Analysis Parameter (a)",
          "Dimension Analysis Parameter (b)",
          "Dimension Analysis Parameter (c)",
          "Dimension Analysis Parameter (d)",
          "Dimension Analysis Parameter (e)",
          "Dimension Analysis DBH Units",
          "Bole Volume Parameter (b0)",
          "Bole Volume Parameter (b1)",
          "Bole Volume Parameter (b2)",
          "Bole Volume Parameter (b3)",
          "Bole Volume Parameter (b4)",
          "Bole Volume Parameter (b5)",
          "Bole Volume Form Class, 60% - 100%"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for Analysis.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.sMessage);
    }
  }

  /**
   * Tests ValidateData() for data analysis behaviors.
   * @throws ModelException if the validation happens incorrectly.
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Valid volume file
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

      //Valid dimension analysis file
      oManager.ClearCurrentData();
      sFileName = WriteBiomassXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

      //Valid bole volume file
      oManager.ClearCurrentData();
      sFileName = WriteBoleVolumeXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Disturbance validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - volume error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - volume error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - volume error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - bole volume error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = WriteBoleVolumeErrorXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - bole volume error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = WriteBoleVolumeErrorXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    System.out.println("Analysis behaviors ValidateData test was successful.");

  }

  /**
   * Writes a valid file for testing parameter file reading of volume
   * calculator behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteVolumeXMLFile1() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteVolumeCommonStuff(oOut);

    oOut.write("<vo_stumpHeight>13</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>3</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>2</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for testing parameter file reading of bole volume
   * calculator behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteBoleVolumeXMLFile1() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteBoleVolumeCommonStuff(oOut);

    oOut.write("<an_boleFormClasses>");
    oOut.write("<an_bfcVal species=\"Species_2\">80</an_bfcVal>");
    oOut.write("<an_bfcVal species=\"Species_3\">79</an_bfcVal>");
    oOut.write("</an_boleFormClasses>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for testing parameter file reading of biomass
   * calculator behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteBiomassXMLFile1() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteBiomassCommonStuff(oOut);
    oOut.write("<analysis>");
    oOut.write("<bi_a>");
    oOut.write("<bi_aVal species=\"Species_1\">0.1</bi_aVal>");
    oOut.write("<bi_aVal species=\"Species_2\">2.4</bi_aVal>");
    oOut.write("<bi_aVal species=\"Species_3\">10</bi_aVal>");
    oOut.write("</bi_a>");
    oOut.write("<bi_c>");
    oOut.write("<bi_cVal species=\"Species_1\">-0.2</bi_cVal>");
    oOut.write("<bi_cVal species=\"Species_2\">0.03</bi_cVal>");
    oOut.write("<bi_cVal species=\"Species_3\">2</bi_cVal>");
    oOut.write("</bi_c>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the stump height is a negative number.  Tests
   * volume calculator behavior validation.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteVolumeXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteVolumeCommonStuff(oOut);

    oOut.write("<vo_stumpHeight>-10</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>5</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>1</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the minimum usable diameter is a negative number.  Tests
   * volume calculator behavior validation.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteVolumeXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteVolumeCommonStuff(oOut);

    oOut.write("<vo_stumpHeight>0</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>-5</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>1</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the segment length is a negative number.  Tests
   * volume calculator behavior validation.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteVolumeXMLErrorFile3() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteVolumeCommonStuff(oOut);

    oOut.write("<vo_stumpHeight>0</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>5</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>-1</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error file for testing parameter file reading of bole volume
   * calculator behavior parameters.  This has a form class of less than 60.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteBoleVolumeErrorXMLFile1() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteBoleVolumeCommonStuff(oOut);

    oOut.write("<an_boleFormClasses>");
    oOut.write("<an_bfcVal species=\"Species_2\">80</an_bfcVal>");
    oOut.write("<an_bfcVal species=\"Species_3\">49</an_bfcVal>");
    oOut.write("</an_boleFormClasses>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error file for testing parameter file reading of bole volume
   * calculator behavior parameters.  This has a form class of more than 100.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteBoleVolumeErrorXMLFile2() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteBoleVolumeCommonStuff(oOut);

    oOut.write("<an_boleFormClasses>");
    oOut.write("<an_bfcVal species=\"Species_2\">80</an_bfcVal>");
    oOut.write("<an_bfcVal species=\"Species_3\">149</an_bfcVal>");
    oOut.write("</an_boleFormClasses>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes common stuff for parameter files validating the volume calculator
   * behavior.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing to the file.
   */
  private void WriteVolumeCommonStuff(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">45</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree volume calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<volume>");
    oOut.write("<vo_taperA>");
    oOut.write("<vo_taVal species=\"Species_2\">0.843142</vo_taVal>");
    oOut.write("<vo_taVal species=\"Species_3\">0.8</vo_taVal>");
    oOut.write("</vo_taperA>");
    oOut.write("<vo_taperB>");
    oOut.write("<vo_tbVal species=\"Species_2\">1.00813</vo_tbVal>");
    oOut.write("<vo_tbVal species=\"Species_3\">0.9</vo_tbVal>");
    oOut.write("</vo_taperB>");
    oOut.write("<vo_taperC>");
    oOut.write("<vo_tcVal species=\"Species_2\">0.031402</vo_tcVal>");
    oOut.write("<vo_tcVal species=\"Species_3\">0.032</vo_tcVal>");
    oOut.write("</vo_taperC>");
    oOut.write("<vo_taperD>");
    oOut.write("<vo_tdVal species=\"Species_2\">0.319773</vo_tdVal>");
    oOut.write("<vo_tdVal species=\"Species_3\">0.321</vo_tdVal>");
    oOut.write("</vo_taperD>");
    oOut.write("<vo_taperF>");
    oOut.write("<vo_tfVal species=\"Species_2\">-0.532732</vo_tfVal>");
    oOut.write("<vo_tfVal species=\"Species_3\">-0.51</vo_tfVal>");
    oOut.write("</vo_taperF>");
    oOut.write("<vo_taperG>");
    oOut.write("<vo_tgVal species=\"Species_2\">0.585361</vo_tgVal>");
    oOut.write("<vo_tgVal species=\"Species_3\">0.59</vo_tgVal>");
    oOut.write("</vo_taperG>");
    oOut.write("<vo_taperI>");
    oOut.write("<vo_tiVal species=\"Species_2\">1.0315</vo_tiVal>");
    oOut.write("<vo_tiVal species=\"Species_3\">1.04</vo_tiVal>");
    oOut.write("</vo_taperI>");
    oOut.write("<vo_taperJ>");
    oOut.write("<vo_tjVal species=\"Species_2\">0.018622</vo_tjVal>");
    oOut.write("<vo_tjVal species=\"Species_3\">0.02</vo_tjVal>");
    oOut.write("</vo_taperJ>");
    oOut.write("<vo_taperK>");
    oOut.write("<vo_tkVal species=\"Species_2\">-0.225146</vo_tkVal>");
    oOut.write("<vo_tkVal species=\"Species_3\">-0.23</vo_tkVal>");
    oOut.write("</vo_taperK>");
    oOut.write("<vo_barkA>");
    oOut.write("<vo_baVal species=\"Species_2\">0.2</vo_baVal>");
    oOut.write("<vo_baVal species=\"Species_3\">0.01</vo_baVal>");
    oOut.write("</vo_barkA>");
    oOut.write("<vo_barkB>");
    oOut.write("<vo_bbVal species=\"Species_2\">0.8</vo_bbVal>");
    oOut.write("<vo_bbVal species=\"Species_3\">1</vo_bbVal>");
    oOut.write("</vo_barkB>");
    oOut.write("<vo_barkC>");
    oOut.write("<vo_bcVal species=\"Species_2\">0.01</vo_bcVal>");
    oOut.write("<vo_bcVal species=\"Species_3\">0.001</vo_bcVal>");
    oOut.write("</vo_barkC>");
  }

  /**
   * Writes common stuff for parameter files validating the bole volume
   * calculator behavior.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing to the file.
   */
  private void WriteBoleVolumeCommonStuff(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">45</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree bole volume calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<analysis>");
    oOut.write("<an_boleB0>");
    oOut.write("<an_bb0Val species=\"Species_2\">0.11</an_bb0Val>");
    oOut.write("<an_bb0Val species=\"Species_3\">0.24</an_bb0Val>");
    oOut.write("</an_boleB0>");
    oOut.write("<an_boleB1>");
    oOut.write("<an_bb1Val species=\"Species_2\">-0.05977</an_bb1Val>");
    oOut.write("<an_bb1Val species=\"Species_3\">-0.05895</an_bb1Val>");
    oOut.write("</an_boleB1>");
    oOut.write("<an_boleB2>");
    oOut.write("<an_bb2Val species=\"Species_2\">2.0498</an_bb2Val>");
    oOut.write("<an_bb2Val species=\"Species_3\">2.0362</an_bb2Val>");
    oOut.write("</an_boleB2>");
    oOut.write("<an_boleB3>");
    oOut.write("<an_bb3Val species=\"Species_2\">0.04965</an_bb3Val>");
    oOut.write("<an_bb3Val species=\"Species_3\">0.04947</an_bb3Val>");
    oOut.write("</an_boleB3>");
    oOut.write("<an_boleB4>");
    oOut.write("<an_bb4Val species=\"Species_2\">2.0198</an_bb4Val>");
    oOut.write("<an_bb4Val species=\"Species_3\">2.0172</an_bb4Val>");
    oOut.write("</an_boleB4>");
    oOut.write("<an_boleB5>");
    oOut.write("<an_bb5Val species=\"Species_2\">0.3468</an_bb5Val>");
    oOut.write("<an_bb5Val species=\"Species_3\">0.3366</an_bb5Val>");
    oOut.write("</an_boleB5>");
  }

  /**
   * Writes common stuff for parameter files validating the biomass calculator
   * behavior.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing to the file.
   */
  private void WriteBiomassCommonStuff(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">45</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree biomass calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  /**
   * Tests a table set against a list of expected members in the table.  It
   * does not care how many tables are defined in the vector.  It will
   * throw a fail if there are members missing or members that are not
   * expected.  Species are not checked.  Blanks are skipped.
   * @param p_oTablesToTest Vector of tables to test.
   * @param p_sExpected Expected member list.
   */
  private void TestTable(Vector p_oTablesToTest, String[] p_sExpected) {
    Object[][] p_oData;
    Vector p_oOneTable;
    String sMemberName;
    int[] p_iFoundTimes = new int[p_sExpected.length]; //holds the number of
    //times each member was found
    int iNumberOfTables = p_oTablesToTest.size(),
        i, j, k;
    boolean bFound;

    //Initialize the number of times found array to all 0s
    for (i = 0; i < p_iFoundTimes.length; i++) {
      p_iFoundTimes[i] = 0;
    }

    //Analyze each table
    for (i = 0; i < iNumberOfTables; i++) {
      p_oOneTable = (Vector) p_oTablesToTest.elementAt(i);

      //The second element in the vector is the data
      p_oData = (Object[][]) p_oOneTable.elementAt(1);

      //Go through each table row
      for (j = 0; j < p_oData.length; j++) {

        sMemberName = (String) p_oData[j][0];
        bFound = false;

        if (!sMemberName.equals(" ")) {

          //Does it match a member of the expected list?
          for (k = 0; k < p_sExpected.length; k++) {

            if (sMemberName.equals(p_sExpected[k])) {

              bFound = true;
              p_iFoundTimes[k]++;

            }
          }
        }

        //Did we find our member in the expected list?
        if (!bFound) {
          fail(sMemberName + " unexpectedly found in table.");
        }
      }
    }

    //Now look at the number of times we found each expected member - if
    //not 1 for each, fail
    for (i = 0; i < p_iFoundTimes.length; i++) {
      assertEquals("Found " + p_sExpected[i] + " unexpected number of times.",
                   1, p_iFoundTimes[i]);
    }
  }

}
