package javawrapper;

import junit.framework.*;

/**
 * Tests the DetailedOutputSettings class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

 */
public class TestDetailedOutputSettings
    extends TestCase {
  private DetailedOutputSettings m_oDetailedOutputSettings = null;

  protected void setUp() throws Exception {
    super.setUp();
    m_oDetailedOutputSettings = new DetailedOutputSettings();
    assertEquals(0, m_oDetailedOutputSettings.GetNumberOfBools());
    assertEquals(0, m_oDetailedOutputSettings.GetNumberOfChars());
    assertEquals(0, m_oDetailedOutputSettings.GetNumberOfFloats());
    assertEquals(0, m_oDetailedOutputSettings.GetNumberOfInts());
    assertEquals(0, m_oDetailedOutputSettings.GetSaveFrequency());
  }

  protected void tearDown() throws Exception {
    m_oDetailedOutputSettings = null;
    super.tearDown();
  }

  public void testEverything() {
    String sLabel = "test bool 1";
    m_oDetailedOutputSettings.AddBool(sLabel);
    assertEquals(1, m_oDetailedOutputSettings.GetNumberOfBools());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetBool(0).GetCodeName());

   sLabel = "test char 1";
    m_oDetailedOutputSettings.AddChar(sLabel);
    assertEquals(1, m_oDetailedOutputSettings.GetNumberOfChars());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetChar(0).GetCodeName());

    sLabel = "test char 2";
    m_oDetailedOutputSettings.AddChar(sLabel);
    assertEquals(2, m_oDetailedOutputSettings.GetNumberOfChars());
    assertEquals("test char 1", m_oDetailedOutputSettings.GetChar(0).GetCodeName());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetChar(1).GetCodeName());

    sLabel = "test float 1";
    m_oDetailedOutputSettings.AddFloat(sLabel);
    assertEquals(1, m_oDetailedOutputSettings.GetNumberOfFloats());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetFloat(0).GetCodeName());

    sLabel = "test float 2";
    m_oDetailedOutputSettings.AddFloat(sLabel);
    assertEquals(2, m_oDetailedOutputSettings.GetNumberOfFloats());
    assertEquals("test float 1", m_oDetailedOutputSettings.GetFloat(0).GetCodeName());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetFloat(1).GetCodeName());

    sLabel = "test float 3";
    m_oDetailedOutputSettings.AddFloat(sLabel);
    assertEquals(3, m_oDetailedOutputSettings.GetNumberOfFloats());
    assertEquals("test float 1", m_oDetailedOutputSettings.GetFloat(0).GetCodeName());
    assertEquals("test float 2", m_oDetailedOutputSettings.GetFloat(1).GetCodeName());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetFloat(2).GetCodeName());

    sLabel = "test int 1";
    m_oDetailedOutputSettings.AddInt(sLabel);
    assertEquals(1, m_oDetailedOutputSettings.GetNumberOfInts());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetInt(0).GetCodeName());

    sLabel = "test int 2";
    m_oDetailedOutputSettings.AddInt(sLabel);
    assertEquals(2, m_oDetailedOutputSettings.GetNumberOfInts());
    assertEquals("test int 1", m_oDetailedOutputSettings.GetInt(0).GetCodeName());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetInt(1).GetCodeName());

    sLabel = "test int 3";
    m_oDetailedOutputSettings.AddInt(sLabel);
    assertEquals(3, m_oDetailedOutputSettings.GetNumberOfInts());
    assertEquals("test int 1", m_oDetailedOutputSettings.GetInt(0).GetCodeName());
    assertEquals("test int 2", m_oDetailedOutputSettings.GetInt(1).GetCodeName());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetInt(2).GetCodeName());

    sLabel = "test int 4";
    m_oDetailedOutputSettings.AddInt(sLabel);
    assertEquals(4, m_oDetailedOutputSettings.GetNumberOfInts());
    assertEquals("test int 1", m_oDetailedOutputSettings.GetInt(0).GetCodeName());
    assertEquals("test int 2", m_oDetailedOutputSettings.GetInt(1).GetCodeName());
    assertEquals("test int 3", m_oDetailedOutputSettings.GetInt(2).GetCodeName());
    assertEquals(sLabel, m_oDetailedOutputSettings.GetInt(3).GetCodeName());

    m_oDetailedOutputSettings.SetSaveFrequency(5);
    assertEquals(5, m_oDetailedOutputSettings.GetSaveFrequency());

    DetailedOutputSettings oCopyTarget = new DetailedOutputSettings();
    DetailedOutputSettings.CopyData(oCopyTarget, m_oDetailedOutputSettings);
    assertEquals(1, m_oDetailedOutputSettings.GetNumberOfBools());
    assertEquals(2, m_oDetailedOutputSettings.GetNumberOfChars());
    assertEquals(3, m_oDetailedOutputSettings.GetNumberOfFloats());
    assertEquals(4, m_oDetailedOutputSettings.GetNumberOfInts());
    assertEquals(m_oDetailedOutputSettings.GetNumberOfBools(), oCopyTarget.GetNumberOfBools());
    assertEquals(m_oDetailedOutputSettings.GetNumberOfChars(), oCopyTarget.GetNumberOfChars());
    assertEquals(m_oDetailedOutputSettings.GetNumberOfFloats(), oCopyTarget.GetNumberOfFloats());
    assertEquals(m_oDetailedOutputSettings.GetNumberOfInts(), oCopyTarget.GetNumberOfInts());
    assertEquals(m_oDetailedOutputSettings.GetSaveFrequency(), oCopyTarget.GetSaveFrequency());

    //Data member names
   assertEquals(m_oDetailedOutputSettings.GetBool(0).GetCodeName(),
                 oCopyTarget.GetBool(0).GetCodeName());
    assertEquals(m_oDetailedOutputSettings.GetFloat(0).GetDisplayName(),
                 oCopyTarget.GetFloat(0).GetDisplayName());
    assertEquals(m_oDetailedOutputSettings.GetInt(0).GetDisplayName(),
                 oCopyTarget.GetInt(0).GetDisplayName());
    assertEquals(m_oDetailedOutputSettings.GetChar(0).GetDisplayName(),
                 oCopyTarget.GetChar(0).GetDisplayName());

    //Make sure the clone was deep
    DataMember oMember = m_oDetailedOutputSettings.GetBool(0);
    int iSaveCode = oMember.GetCode();
    oMember.SetCode(iSaveCode+4);

    assertEquals(iSaveCode, oCopyTarget.GetBool(0).GetCode());
  }
}
