package javawrapper;

import junit.framework.*;
import java.util.Vector;

/**
 * Tests the DisperseBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

 */
public class TestDisperseBehaviors
    extends TestCase {
  private TreePopulation m_oPop = null;

  public TestDisperseBehaviors(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
    try {
      //Create our tree population
      m_oPop = new TreePopulation(null);
      m_oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});
    }
    catch (ModelException oErr) {
      throw new Exception(oErr.sMessage);
    }

  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  public void testFormatDataForDisplay() {
    try {
      DisperseBehaviors oDisperse = null;
      Vector p_oActualReturn;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no disperse behaviors are enabled.
      oDisperse = new DisperseBehaviors(null);
      assertEquals(null, oDisperse.FormatDataForDisplay(m_oPop));

      //Case:  only non-spatial disperse is enabled.
      oDisperse = null;
      oDisperse = new DisperseBehaviors(null);
      oDisperse.DoSetup(m_oPop);
      oBeh = oDisperse.GetBehaviorByKey("non spatial disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Slope Mean Non-Spatial Seed Rain, seeds/m2/ha of BA/yr",
          "Intercept of Mean Non-Spatial Seed Rain, seeds/m2/yr",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  no gap disperse is enabled.
      oDisperse = null;
      oDisperse = new DisperseBehaviors(null);
      oDisperse.DoSetup(m_oPop);
      oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "STR for Stumps",
          "Beta for Stumps",
          "Lognormal Canopy Annual STR",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Weibull Canopy Annual STR",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only gap disperse is enabled.
      oDisperse = null;
      oDisperse = new DisperseBehaviors(null);
      oDisperse.DoSetup(m_oPop);
      oBeh = oDisperse.GetBehaviorByKey("gap disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Weibull Gap Annual STR",
          "Weibull Gap Beta",
          "Weibull Gap Theta",
          "Weibull Gap Dispersal",
          "Maximum Parent Trees Allowed in Gap Cell",
          "Lognormal Gap Annual STR",
          "Lognormal Gap Beta",
          "Lognormal Gap Xb",
          "Lognormal Gap X0",
          "Lognormal Canopy Annual STR",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Gap Function Used",
          "STR for Stumps",
          "Beta for Stumps",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "Weibull Canopy Annual STR",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  all are enabled.
      oDisperse = null;
      oDisperse = new DisperseBehaviors(null);
      oDisperse.DoSetup(m_oPop);
      int i;
      for (i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
        oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oDisperse.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
            SpeciesTypeCombo(
            0, 3, m_oPop));
      }
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Maximum Parent Trees Allowed in Gap Cell",
          "Weibull Gap Annual STR",
          "Weibull Gap Beta",
          "Weibull Gap Theta",
          "Weibull Gap Dispersal",
          "Lognormal Gap Annual STR",
          "Lognormal Gap Beta",
          "Lognormal Gap Xb",
          "Lognormal Gap X0",
          "Lognormal Canopy Annual STR",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Gap Function Used",
          "STR for Stumps",
          "Beta for Stumps",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "Weibull Canopy Annual STR",
          "Slope Mean Non-Spatial Seed Rain, seeds/m2/ha of BA/yr",
          "Intercept of Mean Non-Spatial Seed Rain, seeds/m2/yr",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution",
          "Azimuth Direction of Max Dispersal Distance, in rad",
          "Amplitude of Anisotropic Effect",
          "Maximum Search Distance for Neighbor Parents, in m"
      };
      TestTable(oDisperse.FormatDataForDisplay(m_oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for disperse.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.sMessage);
    }

  }

  /**
   * Tests a table set against a list of expected members in the table.  It
   * does not care how many tables are defined in the vector.  It will
   * throw a fail if there are members missing or members that are not
   * expected.  Species are not checked.  Blanks are skipped.
   * @param p_oTablesToTest Vector of tables to test.
   * @param p_sExpected Expected member list.
   */
  private void TestTable(Vector p_oTablesToTest, String[] p_sExpected) {
    Object[][] p_oData;
    Vector p_oOneTable;
    String sMemberName;
    int[] p_iFoundTimes = new int[p_sExpected.length]; //holds the number of
    //times each member was found
    int iNumberOfTables = p_oTablesToTest.size(),
        i, j, k;
    boolean bFound;

    //Initialize the number of times found array to all 0s
    for (i = 0; i < p_iFoundTimes.length; i++) {
      p_iFoundTimes[i] = 0;
    }

    //Analyze each table
    for (i = 0; i < iNumberOfTables; i++) {
      p_oOneTable = (Vector) p_oTablesToTest.elementAt(i);

      //The second element in the vector is the data
      p_oData = (Object[][]) p_oOneTable.elementAt(1);

      //Go through each table row
      for (j = 0; j < p_oData.length; j++) {

        sMemberName = (String) p_oData[j][0];
        bFound = false;

        if (!sMemberName.equals(" ")) {

          //Does it match a member of the expected list?
          for (k = 0; k < p_sExpected.length; k++) {

            if (sMemberName.equals(p_sExpected[k])) {

              bFound = true;
              p_iFoundTimes[k]++;

            }
          }
        }

        //Did we find our member in the expected list?
        if (!bFound) {
          fail(sMemberName + " unexpectedly found in table.");
        }
      }
    }

    //Now look at the number of times we found each expected member - if
    //not 1 for each, fail
    for (i = 0; i < p_iFoundTimes.length; i++) {
      assertEquals("Found " + p_sExpected[i] + " unexpected number of times.",
                   1, p_iFoundTimes[i]);
    }
  }
}
