package javawrapper;

import junit.framework.*;
import java.util.Vector;
import java.io.FileWriter;
import java.io.File;

/**
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestEstablishmentBehaviors
    extends TestCase {
  private TreePopulation m_oPop = null;

  protected void setUp() throws Exception {
    super.setUp();
    try {
      //Create our tree population
      m_oPop = new TreePopulation(null);
      m_oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});
    }
    catch (ModelException oErr) {
      throw new Exception(oErr.sMessage);
    }
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  public void testFormatDataForDisplay() {
    try {
      EstablishmentBehaviors oEstablishment = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no establishment behaviors are enabled.
      oEstablishment = new EstablishmentBehaviors(null);
      assertEquals(null, oEstablishment.FormatDataForDisplay(m_oPop));

      //Case:  only establishment is enabled.
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      oBeh = oEstablishment.GetBehaviorByKey("establishment");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          m_oPop));
      assertEquals(null, oEstablishment.FormatDataForDisplay(m_oPop));

      //Case:  only non-gap substrate-dependent seed survival is enabled.
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      oBeh = oEstablishment.GetBehaviorByKey("no gap substrate seed survival");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          m_oPop));
      p_sExpected = new String[] {
          "Fraction Seeds Germinating on Canopy Fresh Logs",
          "Fraction Seeds Germinating on Canopy Decayed Logs",
          "Fraction Seeds Germinating on Canopy Scarified Soil",
          "Fraction Seeds Germinating on Canopy Forest Floor Litter",
          "Fraction Seeds Germinating on Canopy Forest Floor Moss",
          "Fraction Seeds Germinating on Canopy Tip-Up"
      };
      TestTable(oEstablishment.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only gap substrate-dependent seed survival is enabled
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      oBeh = oEstablishment.GetBehaviorByKey("gap substrate seed survival");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          m_oPop));
      p_sExpected = new String[] {
          "Fraction Seeds Germinating on Canopy Fresh Logs",
          "Fraction Seeds Germinating on Canopy Decayed Logs",
          "Fraction Seeds Germinating on Canopy Scarified Soil",
          "Fraction Seeds Germinating on Canopy Forest Floor Litter",
          "Fraction Seeds Germinating on Canopy Forest Floor Moss",
          "Fraction Seeds Germinating on Canopy Tip-Up",
          "Fraction Seeds Germinating on Gap Fresh Logs",
          "Fraction Seeds Germinating on Gap Decayed Logs",
          "Fraction Seeds Germinating on Gap Scarified Soil",
          "Fraction Seeds Germinating on Gap Forest Floor Litter",
          "Fraction Seeds Germinating on Gap Forest Floor Moss",
          "Fraction Seeds Germinating on Gap Tip-Up"
      };
      TestTable(oEstablishment.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only microtopographic substrate-dependent seed survival is
      //enabled
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      oBeh = oEstablishment.GetBehaviorByKey("micro substrate seed survival");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          m_oPop));
      p_sExpected = new String[] {
          "Fraction Seeds Germinating on Mound Fresh Logs",
          "Fraction Seeds Germinating on Mound Decayed Logs",
          "Fraction Seeds Germinating on Mound Scarified Soil",
          "Fraction Seeds Germinating on Mound Forest Floor Litter",
          "Fraction Seeds Germinating on Mound Forest Floor Moss",
          "Fraction Seeds Germinating on Mound Tip-Up",
          "Fraction Seeds Germinating on Ground Fresh Logs",
          "Fraction Seeds Germinating on Ground Decayed Logs",
          "Fraction Seeds Germinating on Ground Scarified Soil",
          "Fraction Seeds Germinating on Ground Forest Floor Litter",
          "Fraction Seeds Germinating on Ground Forest Floor Moss",
          "Fraction Seeds Germinating on Ground Tip-Up",
          "Proportion of Plot Area that is Mound"
      };
      TestTable(oEstablishment.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only light-dependent seed survival is enabled.
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      oBeh = oEstablishment.GetBehaviorByKey("light seed survival");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          m_oPop));
      p_sExpected = new String[] {
          "GLI of Optimum Establishment, 0-100",
          "Slope of Dropoff Below Optimum GLI",
          "Slope of Dropoff Above the Optimum GLI",
          "Light Extinction Coeff of Undamaged Trees (0-1)",
          "Light Extinction Coeff of Medium Damage Trees (0-1)",
          "Light Extinction Coeff of Complete Damage Trees (0-1)",
          "Snag Age Class 1 Light Extinction Coefficient (0-1)",
          "Snag Age Class 2 Light Extinction Coefficient (0-1)",
          "Snag Age Class 3 Light Extinction Coefficient (0-1)",
          "Height in m At Which to Calculate GLI",
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "Number of Azimuth Sky Divisions for GLI Light Calculations",
          "Number of Altitude Sky Divisions for GLI Light Calculations",
          "First Day of Growing Season for GLI Light Calculations",
          "Minimum Solar Angle for GLI Calculations, in rad",
          "Upper Age (Yrs) of Snag Light Extinction Class 1",
          "Upper Age (Yrs) of Snag Light Extinction Class 2",
          "Last Day of Growing Season for GLI Light Calculations",
      };
      TestTable(oEstablishment.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only density dependent seed survival is enabled
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      oBeh = oEstablishment.GetBehaviorByKey("density seed survival");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          m_oPop));
      p_sExpected = new String[] {
          "Slope of Density Dependence",
          "Steepness of Density Dependence"
      };
      TestTable(oEstablishment.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only proportional seed survival is enabled
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      oBeh = oEstablishment.GetBehaviorByKey("germination");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          m_oPop));
      p_sExpected = new String[] {
          "Proportion Germinating Between 0 and 1"
      };
      TestTable(oEstablishment.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only microtopographic seed establishment is enabled
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      oBeh = oEstablishment.GetBehaviorByKey("micro establishment");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          m_oPop));
      p_sExpected = new String[] {
          "Proportion of Plot Area that is Mound",
          "Mean Height of Mounds, in m",
          "Standard Deviation of Mound Height, in m",
          "Mean Height of Fresh Log Substrate, in m",
          "Standard Deviation of Fresh Log Substrate Height, in m",
          "# Years Respite from Fern Shading for Seeds on Fresh Logs"
      };
      TestTable(oEstablishment.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  all are enabled.
      oEstablishment = null;
      oEstablishment = new EstablishmentBehaviors(null);
      oEstablishment.DoSetup(m_oPop);
      int i;
      for (i = 0; i < oEstablishment.mp_oChildBehaviors.length; i++) {
        oEstablishment.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oEstablishment.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
            SpeciesTypeCombo(0, TreePopulation.SEED,
                             m_oPop));
      }
      p_sExpected = new String[] {
          "Fraction Seeds Germinating on Canopy Fresh Logs",
          "Fraction Seeds Germinating on Canopy Decayed Logs",
          "Fraction Seeds Germinating on Canopy Scarified Soil",
          "Fraction Seeds Germinating on Canopy Forest Floor Litter",
          "Fraction Seeds Germinating on Canopy Forest Floor Moss",
          "Fraction Seeds Germinating on Canopy Tip-Up",
          "Fraction Seeds Germinating on Gap Fresh Logs",
          "Fraction Seeds Germinating on Gap Decayed Logs",
          "Fraction Seeds Germinating on Gap Scarified Soil",
          "Fraction Seeds Germinating on Gap Forest Floor Litter",
          "Fraction Seeds Germinating on Gap Forest Floor Moss",
          "Fraction Seeds Germinating on Gap Tip-Up",
          "GLI of Optimum Establishment, 0-100",
          "Slope of Dropoff Below Optimum GLI",
          "Slope of Dropoff Above the Optimum GLI",
          "Light Extinction Coeff of Undamaged Trees (0-1)",
          "Light Extinction Coeff of Medium Damage Trees (0-1)",
          "Light Extinction Coeff of Complete Damage Trees (0-1)",
          "Snag Age Class 1 Light Extinction Coefficient (0-1)",
          "Snag Age Class 2 Light Extinction Coefficient (0-1)",
          "Snag Age Class 3 Light Extinction Coefficient (0-1)",
          "Height in m At Which to Calculate GLI",
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "Number of Azimuth Sky Divisions for GLI Light Calculations",
          "Number of Altitude Sky Divisions for GLI Light Calculations",
          "First Day of Growing Season for GLI Light Calculations",
          "Minimum Solar Angle for GLI Calculations, in rad",
          "Upper Age (Yrs) of Snag Light Extinction Class 1",
          "Upper Age (Yrs) of Snag Light Extinction Class 2",
          "Last Day of Growing Season for GLI Light Calculations",
          "Slope of Density Dependence",
          "Steepness of Density Dependence",
          "Proportion Germinating Between 0 and 1",
          "Fraction Seeds Germinating on Mound Fresh Logs",
          "Fraction Seeds Germinating on Mound Decayed Logs",
          "Fraction Seeds Germinating on Mound Scarified Soil",
          "Fraction Seeds Germinating on Mound Forest Floor Litter",
          "Fraction Seeds Germinating on Mound Forest Floor Moss",
          "Fraction Seeds Germinating on Mound Tip-Up",
          "Fraction Seeds Germinating on Ground Fresh Logs",
          "Fraction Seeds Germinating on Ground Decayed Logs",
          "Fraction Seeds Germinating on Ground Scarified Soil",
          "Fraction Seeds Germinating on Ground Forest Floor Litter",
          "Fraction Seeds Germinating on Ground Forest Floor Moss",
          "Fraction Seeds Germinating on Ground Tip-Up",
          "Proportion of Plot Area that is Mound",
          "Mean Height of Mounds, in m",
          "Standard Deviation of Mound Height, in m",
          "Mean Height of Fresh Log Substrate, in m",
          "Standard Deviation of Fresh Log Substrate Height, in m",
          "# Years Respite from Fern Shading for Seeds on Fresh Logs"
      };
      //For some reason this isn't working.  I've checked it by hand and it's
      //right.
      TestTable(oEstablishment.FormatDataForDisplay(m_oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for disperse.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.sMessage);
    }

  }

  /**
   * Tests a table set against a list of expected members in the table.  It
   * does not care how many tables are defined in the vector.  It will
   * throw a fail if there are members missing or members that are not
   * expected.  Species are not checked.  Blanks are skipped.
   * @param p_oTablesToTest Vector of tables to test.
   * @param p_sExpected Expected member list.
   */
  private void TestTable(Vector p_oTablesToTest, String[] p_sExpected) {
    Object[][] p_oData;
    Vector p_oOneTable;
    String sMemberName;
    int[] p_iFoundTimes = new int[p_sExpected.length]; //holds the number of
    //times each member was found
    int iNumberOfTables = p_oTablesToTest.size(),
        i, j, k;
    boolean bFound;

    //Initialize the number of times found array to all 0s
    for (i = 0; i < p_iFoundTimes.length; i++) {
      p_iFoundTimes[i] = 0;
    }

    //Analyze each table
    for (i = 0; i < iNumberOfTables; i++) {
      p_oOneTable = (Vector) p_oTablesToTest.elementAt(i);

      //The second element in the vector is the data
      p_oData = (Object[][]) p_oOneTable.elementAt(1);

      //Go through each table row
      for (j = 0; j < p_oData.length; j++) {

        sMemberName = (String) p_oData[j][0];
        bFound = false;

        if (!sMemberName.equals(" ")) {

          //Does it match a member of the expected list?
          for (k = 0; k < p_sExpected.length; k++) {

            if (sMemberName.equals(p_sExpected[k])) {

              bFound = true;
              p_iFoundTimes[k]++;

            }
          }
        }

        //Did we find our member in the expected list?
        if (!bFound) {
          fail(sMemberName + " unexpectedly found in table.");
        }
      }
    }

    //Now look at the number of times we found each expected member - if
    //not 1 for each, fail
    for (i = 0; i < p_iFoundTimes.length; i++) {
      assertEquals("Found " + p_sExpected[i] + " unexpected number of times.",
                   1, p_iFoundTimes[i]);
    }
  }

  /**
   * Makes sure all parameters read correctly.
   */
  public void testParFileReading() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Valid PR file 1
      oManager.ClearCurrentData();
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      EstablishmentBehaviors oEst = oManager.GetEstablishmentBehaviors();

      assertEquals( ( (Float) oEst.mp_fDensDepSlope.GetValue().elementAt(0)).
                   floatValue(), 0.4, 0.001);
      assertEquals( ( (Float) oEst.mp_fDensDepSlope.GetValue().elementAt(1)).
                   floatValue(), 0.2, 0.001);

      assertEquals( ( (Float) oEst.mp_fDensDepSteepness.GetValue().elementAt(
          0)).floatValue(), -0.1, 0.001);
      assertEquals( ( (Float) oEst.mp_fDensDepSteepness.GetValue().elementAt(
          1)).floatValue(), -0.2, 0.001);

      assertEquals( ( (Float) oEst.mp_fProportionGerminating.GetValue().
                     elementAt(
          0)).floatValue(), 0.46, 0.001);
      assertEquals( ( (Float) oEst.mp_fProportionGerminating.GetValue().
                     elementAt(
          1)).floatValue(), 0.68, 0.001);

      assertEquals( ( (Float) oEst.mp_fOptimumGLI.GetValue().elementAt(0)).
                   floatValue(), 87, 0.001);
      assertEquals( ( (Float) oEst.mp_fOptimumGLI.GetValue().elementAt(1)).
                   floatValue(), 23, 0.001);

      assertEquals( ( (Float) oEst.mp_fHighSlope.GetValue().elementAt(0)).
                   floatValue(), 0.0098, 0.001);
      assertEquals( ( (Float) oEst.mp_fHighSlope.GetValue().elementAt(1)).
                   floatValue(), 0.0023, 0.001);

      assertEquals( ( (Float) oEst.mp_fLowSlope.GetValue().elementAt(0)).
                   floatValue(), 0.0098, 0.001);
      assertEquals( ( (Float) oEst.mp_fLowSlope.GetValue().elementAt(1)).
                   floatValue(), 0.034, 0.001);

      assertEquals( ( (Float) oEst.mp_fLightExtCoeffNoDmg.GetValue().
                     elementAt(0)).floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oEst.mp_fLightExtCoeffNoDmg.GetValue().
                     elementAt(1)).floatValue(), 0.21, 0.001);

      assertEquals( ( (Float) oEst.mp_fLightExtCoeffPartDmg.GetValue().
                     elementAt(0)).floatValue(), 0.22, 0.001);
      assertEquals( ( (Float) oEst.mp_fLightExtCoeffPartDmg.GetValue().
                     elementAt(1)).floatValue(), 0.23, 0.001);

      assertEquals( ( (Float) oEst.mp_fLightExtCoeffFullDmg.GetValue().
                     elementAt(0)).floatValue(), 0.9, 0.001);
      assertEquals( ( (Float) oEst.mp_fLightExtCoeffFullDmg.GetValue().
                     elementAt(1)).floatValue(), 0.8, 0.001);

      assertEquals( ( (Float) oEst.mp_fSnagClass1LightExtinctionCoefficient.
                     GetValue().
                     elementAt(0)).floatValue(), 0.6, 0.001);
      assertEquals( ( (Float) oEst.mp_fSnagClass1LightExtinctionCoefficient.
                     GetValue().
                     elementAt(1)).floatValue(), 0.7, 0.001);

      assertEquals( ( (Float) oEst.mp_fSnagClass2LightExtinctionCoefficient.
                     GetValue().
                     elementAt(0)).floatValue(), 0.4, 0.001);
      assertEquals( ( (Float) oEst.mp_fSnagClass2LightExtinctionCoefficient.
                     GetValue().
                     elementAt(1)).floatValue(), 0.5, 0.001);

      assertEquals( ( (Float) oEst.mp_fSnagClass3LightExtinctionCoefficient.
                     GetValue().
                     elementAt(0)).floatValue(), 1, 0.001);
      assertEquals( ( (Float) oEst.mp_fSnagClass3LightExtinctionCoefficient.
                     GetValue().
                     elementAt(1)).floatValue(), 0.1, 0.001);

      assertEquals(oEst.m_fLightHeight.GetValue(), (float) 2, 0.001);
      assertEquals(oEst.m_fBeamFractionOfGlobalRadiation.GetValue(), 0.5, 0.001);
      assertEquals(oEst.m_fClearSkyTransmissionCoefficient.GetValue(), 0.65,
                   0.001);
      assertEquals(oEst.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oEst.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oEst.m_fMinSunAngle.GetValue(), 0.785, 0.001);
      assertEquals(oEst.m_iNumAltDiv.GetValue(), 12);
      assertEquals(oEst.m_iNumAziDiv.GetValue(), 18);

      assertEquals(oEst.m_iSnagAgeClass1.GetValue(), 10);
      assertEquals(oEst.m_iSnagAgeClass2.GetValue(), 17);

      assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                   floatValue(), 0.058, 0.001);
      assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                   floatValue(), 0.024, 0.001);

      assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(0)).floatValue(),
                   0.7, 0.001);
      assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(1)).floatValue(),
                   0.048, 0.001);

      assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                   floatValue(), 0.0090, 0.001);
      assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                   floatValue(), 0.01, 0.001);

      assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                   floatValue(), 0.162, 0.001);
      assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                   floatValue(), 0.885, 0.001);

      assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(0)).floatValue(),
                   0.894, 0.001);
      assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(1)).floatValue(),
                   0.0010, 0.001);

      assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(0)).floatValue(),
                   0.568, 0.001);
      assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(1)).floatValue(),
                   0.911, 0.001);

      assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[DisperseBehaviors.
                     CANOPY].GetValue().elementAt(0)).floatValue(), 0.353,
                   0.001);
      assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[DisperseBehaviors.
                     CANOPY].GetValue().elementAt(1)).floatValue(), 0.424,
                   0.001);

      assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[DisperseBehaviors.
                     GAP].GetValue().elementAt(0)).floatValue(), 0.967, 0.001);
      assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[DisperseBehaviors.
                     GAP].GetValue().elementAt(1)).floatValue(), 0.319, 0.001);

      assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                   floatValue(), 0.129, 0.001);
      assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                   floatValue(), 0.48, 0.001);

      assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(0)).floatValue(),
                   0.925, 0.001);
      assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(1)).floatValue(),
                   0.213, 0.001);

      assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.
                     CANOPY].GetValue().elementAt(0)).floatValue(), 0.983,
                   0.001);
      assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.
                     CANOPY].GetValue().elementAt(1)).floatValue(), 0.391,
                   0.001);

      assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.GAP].
                     GetValue().elementAt(0)).floatValue(), 0.913, 0.001);
      assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.GAP].
                     GetValue().elementAt(1)).floatValue(), 0.179, 0.001);

      assertEquals(oEst.m_fMoundProportion.GetValue(), 0.54, 0.001);

      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Establishment validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests to make sure that we maintain backwards compatibility
   */
  public void testBackwardsCompatibility() {
    GUIManager oManager = null;
    String sFileName = null;
    //****************************************
     // Test the changes introduced for 6.04
     //***************************************
    try {

      oManager = new GUIManager(null);

      //Gap substrate dependent seed survival
      oManager.ClearCurrentData();
      sFileName = WriteOldGapSubstrateFile();
      oManager.InputXMLParameterFile(sFileName);
      EstablishmentBehaviors oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());

      //Make sure the right behaviors are enabled
      assertTrue(oEst.GetBehaviorByKey("gap substrate seed survival").
                 m_bIsEnabled);
      assertTrue(oEst.GetBehaviorByKey("establishment").
                 m_bIsEnabled);
      assertFalse(oEst.GetBehaviorByKey("no gap substrate seed survival").
                  m_bIsEnabled);

      //Make sure the values are right
      assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                   floatValue(), 0.058, 0.001);
      assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                   floatValue(), 0.024, 0.001);

      assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(0)).floatValue(),
                   0.7, 0.001);
      assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(1)).floatValue(),
                   0.048, 0.001);

      assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                   floatValue(), 0.0090, 0.001);
      assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                   floatValue(), 0.01, 0.001);

      assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                   floatValue(), 0.0090, 0.001);
      assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                   floatValue(), 0.01, 0.001);

      assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(0)).floatValue(),
                   0.894, 0.001);
      assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(1)).floatValue(),
                   0.0010, 0.001);

      assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(0)).floatValue(),
                   0.894, 0.001);
      assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(1)).floatValue(),
                   0.0010, 0.001);

      assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[DisperseBehaviors.
                     CANOPY].GetValue().elementAt(0)).floatValue(), 0.353,
                   0.001);
      assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[DisperseBehaviors.
                     CANOPY].GetValue().elementAt(1)).floatValue(), 0.424,
                   0.001);

      assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[DisperseBehaviors.
                     GAP].GetValue().elementAt(0)).floatValue(), 0.967, 0.001);
      assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[DisperseBehaviors.
                     GAP].GetValue().elementAt(1)).floatValue(), 0.319, 0.001);

      assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                   floatValue(), 0.129, 0.001);
      assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                     DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                   floatValue(), 0.48, 0.001);

      assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(0)).floatValue(),
                   0.925, 0.001);
      assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                     DisperseBehaviors.GAP].GetValue().elementAt(1)).floatValue(),
                   0.213, 0.001);

      assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.
                     CANOPY].GetValue().elementAt(0)).floatValue(), 0.983,
                   0.001);
      assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.
                     CANOPY].GetValue().elementAt(1)).floatValue(), 0.391,
                   0.001);

      assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.GAP].
                     GetValue().elementAt(0)).floatValue(), 0.913, 0.001);
      assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.GAP].
                     GetValue().elementAt(1)).floatValue(), 0.179, 0.001);
    }
    catch (ModelException oErr) {
      fail("Establishment backwards compatibility test failed with message " +
           oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {

      oManager = new GUIManager(null);

      //Non-gap substrate dependent seed survival
      oManager.ClearCurrentData();
      sFileName = WriteOldNonGapSubstrateFile();
      oManager.InputXMLParameterFile(sFileName);
      EstablishmentBehaviors oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());

      //Make sure the right behaviors are enabled
      assertFalse(oEst.GetBehaviorByKey("gap substrate seed survival").
                  m_bIsEnabled);
      assertTrue(oEst.GetBehaviorByKey("establishment").
                 m_bIsEnabled);
      assertTrue(oEst.GetBehaviorByKey("no gap substrate seed survival").
                 m_bIsEnabled);

    }
    catch (ModelException oErr) {
      fail("Establishment backwards compatibility test failed with message " +
           oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {

      oManager = new GUIManager(null);

      //Non-gap non-substrate dependent seed survival
      oManager.ClearCurrentData();
      sFileName = WriteOldNonGapNonSubstrateFile();
      oManager.InputXMLParameterFile(sFileName);
      EstablishmentBehaviors oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());

      //Make sure the right behaviors are enabled
      assertFalse(oEst.GetBehaviorByKey("gap substrate seed survival").
                  m_bIsEnabled);
      assertTrue(oEst.GetBehaviorByKey("establishment").
                 m_bIsEnabled);
      assertFalse(oEst.GetBehaviorByKey("no gap substrate seed survival").
                  m_bIsEnabled);

    }
    catch (ModelException oErr) {
      fail("Establishment backwards compatibility test failed with message " +
           oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //****************************************
     // Test the changes introduced for 6.05
     //***************************************
      try {

        oManager = new GUIManager(null);

        oManager.ClearCurrentData();
        sFileName = WriteVersion6_04File();
        oManager.InputXMLParameterFile(sFileName);
        EstablishmentBehaviors oEst = oManager.GetEstablishmentBehaviors();
        oEst.ValidateData(oManager.GetTreePopulation());

        //Make sure the right behaviors are enabled
        assertTrue(oEst.GetBehaviorByKey("gap substrate seed survival").
                   m_bIsEnabled);
        assertTrue(oEst.GetBehaviorByKey("no gap substrate seed survival").
                   m_bIsEnabled);

        //Make sure the values are right
        assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                       DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                     floatValue(), 0.058, 0.001);
        assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                       DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                     floatValue(), 0.024, 0.001);

        assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                       DisperseBehaviors.GAP].GetValue().elementAt(0)).
                     floatValue(),
                     0.7, 0.001);
        assertEquals( ( (Float) oEst.mp_fDecayedLogsFavorability[
                       DisperseBehaviors.GAP].GetValue().elementAt(1)).
                     floatValue(),
                     0.048, 0.001);

        assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                       DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                     floatValue(), 0.0090, 0.001);
        assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                       DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                     floatValue(), 0.01, 0.001);

        assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                       DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                     floatValue(), 0.0090, 0.001);
        assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                       DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                     floatValue(), 0.01, 0.001);

        assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                       DisperseBehaviors.GAP].GetValue().elementAt(0)).
                     floatValue(),
                     0.894, 0.001);
        assertEquals( ( (Float) oEst.mp_fForestFloorLitterFavorability[
                       DisperseBehaviors.GAP].GetValue().elementAt(1)).
                     floatValue(),
                     0.0010, 0.001);

        assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                       DisperseBehaviors.GAP].GetValue().elementAt(0)).
                     floatValue(),
                     0.894, 0.001);
        assertEquals( ( (Float) oEst.mp_fForestFloorMossFavorability[
                       DisperseBehaviors.GAP].GetValue().elementAt(1)).
                     floatValue(),
                     0.0010, 0.001);

        assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[
                       DisperseBehaviors.
                       CANOPY].GetValue().elementAt(0)).floatValue(), 0.353,
                     0.001);
        assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[
                       DisperseBehaviors.
                       CANOPY].GetValue().elementAt(1)).floatValue(), 0.424,
                     0.001);

        assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[
                       DisperseBehaviors.
                       GAP].GetValue().elementAt(0)).floatValue(), 0.967, 0.001);
        assertEquals( ( (Float) oEst.mp_fFreshLogsFavorability[
                       DisperseBehaviors.
                       GAP].GetValue().elementAt(1)).floatValue(), 0.319, 0.001);

        assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                       DisperseBehaviors.CANOPY].GetValue().elementAt(0)).
                     floatValue(), 0.129, 0.001);
        assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                       DisperseBehaviors.CANOPY].GetValue().elementAt(1)).
                     floatValue(), 0.48, 0.001);

        assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                       DisperseBehaviors.GAP].GetValue().elementAt(0)).
                     floatValue(),
                     0.925, 0.001);
        assertEquals( ( (Float) oEst.mp_fScarifiedSoilFavorability[
                       DisperseBehaviors.GAP].GetValue().elementAt(1)).
                     floatValue(),
                     0.213, 0.001);

        assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.
                       CANOPY].GetValue().elementAt(0)).floatValue(), 0.983,
                     0.001);
        assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.
                       CANOPY].GetValue().elementAt(1)).floatValue(), 0.391,
                     0.001);

        assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.
                       GAP].
                       GetValue().elementAt(0)).floatValue(), 0.913, 0.001);
        assertEquals( ( (Float) oEst.mp_fTipUpFavorability[DisperseBehaviors.
                       GAP].
                       GetValue().elementAt(1)).floatValue(), 0.179, 0.001);
      }
      catch (ModelException oErr) {
        fail("Establishment backwards compatibility test failed with message " +
             oErr.sMessage);
      }
      catch (java.io.IOException oE) {
        fail("Caught IOException.  Message: " + oE.getMessage());
      }
      finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests ValidateData().
   * @throws ModelException if testing fails
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    EstablishmentBehaviors oEst = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());

      //Make sure establishment gets enabled if it isn't already
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.GetBehaviorByKey("establishment").m_bIsEnabled = false;
      oEst.GetBehaviorByKey("establishment").ClearSpeciesTypeCombos();
      assertFalse(oEst.GetBehaviorByKey("establishment").m_bIsEnabled);
      oEst.ValidateData(oManager.GetTreePopulation());
      assertTrue(oEst.GetBehaviorByKey("establishment").m_bIsEnabled);
    }
    catch (ModelException oErr) {
      fail("Establishment validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Valid microtopography file
    try {

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteValidMicroFile();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());

      //Make sure establishment didn't get enabled as well
      assertFalse(oEst.GetBehaviorByKey("establishment").m_bIsEnabled);
    }
    catch (ModelException oErr) {
      fail("Establishment validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - microtopographic establishment is enabled but
    //light filter isn't
    try {
      sFileName = WriteValidMicroFile();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oManager.GetLightBehaviors().GetBehaviorByKey("light filter").m_bIsEnabled = false;
      oManager.GetLightBehaviors().GetBehaviorByKey("light filter").ClearSpeciesTypeCombos();
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch no light filter with microtopographic establishment.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    //Reset
    try {
      sFileName = WriteValidMicroFile();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - microtopographic establishment is enabled but
    //substrate isn't
    try {
      oManager.GetSubstrateBehaviors().GetBehaviorByKey("substrate").m_bIsEnabled = false;
      oManager.GetSubstrateBehaviors().GetBehaviorByKey("substrate").ClearSpeciesTypeCombos();
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch no substrate with microtopographic establishment.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidMicroFile();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in light error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in light error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in light error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 4
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in light error file 4.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 5
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in light error file 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 6
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile6();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 6.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 7
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile7();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 7.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 8
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile8();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 8.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 9
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile9();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 9.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 10
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile10();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 10.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 11
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile11();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 11.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    //Exception processing - error file 12
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile12();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 12.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 13
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile13();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 13.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 14
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile14();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 14.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 15
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile15();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 15.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 16
    try {
      oManager.ClearCurrentData();
      sFileName = LightTestFiles.WriteLightErrorFile16();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in Light error file 16.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 4
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 4.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 5
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 6
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile6();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 6.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 7
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile7();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 7.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 8
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile8();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 8.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 9
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile9();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 9.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 10
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile10();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 10.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 11
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile11();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 11.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - substrate error file 12
    try {
      oManager.ClearCurrentData();
      sFileName = SubstrateTestFiles.WriteSubstrateErrorFile12();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetEstablishmentBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in substrate error file 12.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  mound tip-up isn't a proportion
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.mp_fTipUpFavorability[EstablishmentBehaviors.MOUND].GetValue().clear();
      oEst.mp_fTipUpFavorability[EstablishmentBehaviors.MOUND].GetValue().add(new
          Float(0.2));
      oEst.mp_fTipUpFavorability[EstablishmentBehaviors.MOUND].GetValue().add(new
          Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for mound tip-up favorability.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  mound scarified soil isn't a proportion
    try {
      oEst.mp_fScarifiedSoilFavorability[EstablishmentBehaviors.MOUND].GetValue().
          clear();
      oEst.mp_fScarifiedSoilFavorability[EstablishmentBehaviors.MOUND].GetValue().
          add(new Float(0.2));
      oEst.mp_fScarifiedSoilFavorability[EstablishmentBehaviors.MOUND].GetValue().
          add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for mound scarified soil favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  mound forest floor litter isn't a proportion
    try {
      oEst.mp_fForestFloorLitterFavorability[EstablishmentBehaviors.MOUND].
          GetValue().clear();
      oEst.mp_fForestFloorLitterFavorability[EstablishmentBehaviors.MOUND].
          GetValue().add(new Float(0.2));
      oEst.mp_fForestFloorLitterFavorability[EstablishmentBehaviors.MOUND].
          GetValue().add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail("Establishment didn't catch bad value for mound forest floor litter favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  mound forest floor moss isn't a proportion
    try {
      oEst.mp_fForestFloorMossFavorability[EstablishmentBehaviors.MOUND].
          GetValue().clear();
      oEst.mp_fForestFloorMossFavorability[EstablishmentBehaviors.MOUND].
          GetValue().add(new Float(0.2));
      oEst.mp_fForestFloorMossFavorability[EstablishmentBehaviors.MOUND].
          GetValue().add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for mound forest floor moss favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  mound fresh logs isn't a proportion
    try {
      oEst.mp_fFreshLogsFavorability[EstablishmentBehaviors.MOUND].GetValue().
          clear();
      oEst.mp_fFreshLogsFavorability[EstablishmentBehaviors.MOUND].GetValue().
          add(new Float(0.2));
      oEst.mp_fFreshLogsFavorability[EstablishmentBehaviors.MOUND].GetValue().
          add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for mound fresh logs favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  mound decayed logs isn't a proportion
    try {
      oEst.mp_fDecayedLogsFavorability[EstablishmentBehaviors.MOUND].GetValue().
          clear();
      oEst.mp_fDecayedLogsFavorability[EstablishmentBehaviors.MOUND].GetValue().
          add(new Float(0.2));
      oEst.mp_fDecayedLogsFavorability[EstablishmentBehaviors.MOUND].GetValue().
          add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for mound decayed logs favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  ground tip-up isn't a proportion
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.mp_fTipUpFavorability[EstablishmentBehaviors.GROUND].GetValue().
          clear();
      oEst.mp_fTipUpFavorability[EstablishmentBehaviors.GROUND].GetValue().add(new
          Float(0.2));
      oEst.mp_fTipUpFavorability[EstablishmentBehaviors.GROUND].GetValue().add(new
          Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for ground tip-up favorability.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  ground scarified soil isn't a proportion
    try {
      oEst.mp_fScarifiedSoilFavorability[EstablishmentBehaviors.GROUND].
          GetValue().clear();
      oEst.mp_fScarifiedSoilFavorability[EstablishmentBehaviors.GROUND].
          GetValue().add(new Float(0.2));
      oEst.mp_fScarifiedSoilFavorability[EstablishmentBehaviors.GROUND].
          GetValue().add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for ground scarified soil favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  ground forest floor litter isn't a proportion
    try {
      oEst.mp_fForestFloorLitterFavorability[EstablishmentBehaviors.GROUND].
          GetValue().clear();
      oEst.mp_fForestFloorLitterFavorability[EstablishmentBehaviors.GROUND].
          GetValue().add(new Float(0.2));
      oEst.mp_fForestFloorLitterFavorability[EstablishmentBehaviors.GROUND].
          GetValue().add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail("Establishment didn't catch bad value for ground forest floor litter favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  ground forest floor moss isn't a proportion
    try {
      oEst.mp_fForestFloorMossFavorability[EstablishmentBehaviors.GROUND].
          GetValue().clear();
      oEst.mp_fForestFloorMossFavorability[EstablishmentBehaviors.GROUND].
          GetValue().add(new Float(0.2));
      oEst.mp_fForestFloorMossFavorability[EstablishmentBehaviors.GROUND].
          GetValue().add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail("Establishment didn't catch bad value for ground forest floor moss favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  ground fresh logs isn't a proportion
    try {
      oEst.mp_fFreshLogsFavorability[EstablishmentBehaviors.GROUND].GetValue().
          clear();
      oEst.mp_fFreshLogsFavorability[EstablishmentBehaviors.GROUND].GetValue().
          add(new Float(0.2));
      oEst.mp_fFreshLogsFavorability[EstablishmentBehaviors.GROUND].GetValue().
          add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for ground fresh logs favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - case:  ground decayed logs isn't a proportion
    try {
      oEst.mp_fDecayedLogsFavorability[EstablishmentBehaviors.GROUND].GetValue().
          clear();
      oEst.mp_fDecayedLogsFavorability[EstablishmentBehaviors.GROUND].GetValue().
          add(new Float(0.2));
      oEst.mp_fDecayedLogsFavorability[EstablishmentBehaviors.GROUND].GetValue().
          add(new Float( -1.2));
      oEst.ValidateData(oManager.GetTreePopulation());
      fail(
          "Establishment didn't catch bad value for ground decayed logs favorability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Reset
    try {
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEst = oManager.GetEstablishmentBehaviors();
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Exception processing - case:  mound proportion isn't a proportion
      oEst.m_fMoundProportion.SetValue( (float) 1.5);
      oEst.ValidateData(oManager.GetTreePopulation());
    }
    catch (ModelException oErr) {
      ;
    }
    //Reset
    oEst.m_fMoundProportion.SetValue( (float) 0.5);

    System.out.println("Establishment ValidateData testing succeeded.");
  }

  /**
   * Writes a parameter file using an old-style gap-based substrate
   * establishment.
   * @throws IOException
   * @return String
   */
  private String WriteOldGapSubstrateFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>substrate</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>gap disperse</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>gap and substrate establishment</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<substrate>");
    oOut.write("<su_gapmossDecayAlpha>-5.1E-4</su_gapmossDecayAlpha>");
    oOut.write("<su_gapmossDecayBeta>4.4</su_gapmossDecayBeta>");
    oOut.write("<su_organicDecayAlpha>-7.0E-4</su_organicDecayAlpha>");
    oOut.write("<su_organicDecayBeta>4.3</su_organicDecayBeta>");
    oOut.write("<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>");
    oOut.write("<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>");
    oOut.write("<su_mossyLogToCanMoss>0.55</su_mossyLogToCanMoss>");
    oOut.write("<su_maxNumberDecayYears>50</su_maxNumberDecayYears>");
    oOut.write("<su_initialGapMoss>0.0</su_initialGapMoss>");
    oOut.write("<su_initialOrganic>0.0</su_initialOrganic>");
    oOut.write("<su_initialFreshLog>0.01</su_initialFreshLog>");
    oOut.write("<su_initialMossyLog>0.11</su_initialMossyLog>");
    oOut.write("<su_partialCutGapMoss>0.17</su_partialCutGapMoss>");
    oOut.write("<su_partialCutOrganic>0.15</su_partialCutOrganic>");
    oOut.write("<su_partialCutFreshLog>0.06</su_partialCutFreshLog>");
    oOut.write("<su_partialCutMossyLog>0.04</su_partialCutMossyLog>");
    oOut.write("<su_gapCutGapMoss>0.45</su_gapCutGapMoss>");
    oOut.write("<su_gapCutOrganic>0.34</su_gapCutOrganic>");
    oOut.write("<su_gapCutFreshLog>0.09</su_gapCutFreshLog>");
    oOut.write("<su_gapCutMossyLog>0.0</su_gapCutMossyLog>");
    oOut.write("<su_clearCutGapMoss>0.36</su_clearCutGapMoss>");
    oOut.write("<su_clearCutOrganic>0.34</su_clearCutOrganic>");
    oOut.write("<su_clearCutFreshLog>0.17</su_clearCutFreshLog>");
    oOut.write("<su_clearCutMossyLog>0.01</su_clearCutMossyLog>");
    oOut.write("<su_propOfDeadFall>");
    oOut.write("<su_podfVal species=\"Species_1\">0.7</su_podfVal>");
    oOut.write("<su_podfVal species=\"Species_2\">0.7</su_podfVal>");
    oOut.write("</su_propOfDeadFall>");
    oOut.write("<su_propOfFallUproot>");
    oOut.write("<su_pofuVal species=\"Species_1\">0.5</su_pofuVal>");
    oOut.write("<su_pofuVal species=\"Species_2\">0.5</su_pofuVal>");
    oOut.write("</su_propOfFallUproot>");
    oOut.write("<su_rootTipupFactor>3.1</su_rootTipupFactor>");
    oOut.write("</substrate>");
    oOut.write("<disperse>");
    oOut.write("<di_minDbhForReproduction>");
    oOut.write("<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>");
    oOut.write("<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>");
    oOut.write("</di_minDbhForReproduction>");
    oOut.write("<di_weibullCanopySTR>");
    oOut.write("<di_wcsVal species=\"Species_1\">3.8650002</di_wcsVal>");
    oOut.write("<di_wcsVal species=\"Species_2\">2.4514</di_wcsVal>");
    oOut.write("</di_weibullCanopySTR>");
    oOut.write("<di_weibullCanopyBeta>");
    oOut.write("<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>");
    oOut.write("<di_wcbVal species=\"Species_2\">2.0</di_wcbVal>");
    oOut.write("</di_weibullCanopyBeta>");
    oOut.write("<di_weibullCanopyDispersal>");
    oOut.write("<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>");
    oOut.write("<di_wcdVal species=\"Species_2\">1.82E-4</di_wcdVal>");
    oOut.write("</di_weibullCanopyDispersal>");
    oOut.write("<di_weibullCanopyTheta>");
    oOut.write("<di_wctVal species=\"Species_1\">3.0</di_wctVal>");
    oOut.write("<di_wctVal species=\"Species_2\">3.0</di_wctVal>");
    oOut.write("</di_weibullCanopyTheta>");
    oOut.write("<di_weibullGapSTR>");
    oOut.write("<di_wgsVal species=\"Species_1\">1.2816</di_wgsVal>");
    oOut.write("<di_wgsVal species=\"Species_2\">2.9788</di_wgsVal>");
    oOut.write("</di_weibullGapSTR>");
    oOut.write("<di_weibullGapBeta>");
    oOut.write("<di_wgbVal species=\"Species_1\">2.0</di_wgbVal>");
    oOut.write("<di_wgbVal species=\"Species_2\">2.0</di_wgbVal>");
    oOut.write("</di_weibullGapBeta>");
    oOut.write("<di_weibullGapDispersal>");
    oOut.write("<di_wgdVal species=\"Species_1\">1.14E-6</di_wgdVal>");
    oOut.write("<di_wgdVal species=\"Species_2\">7.46E-5</di_wgdVal>");
    oOut.write("</di_weibullGapDispersal>");
    oOut.write("<di_weibullGapTheta>");
    oOut.write("<di_wgtVal species=\"Species_1\">3.0</di_wgtVal>");
    oOut.write("<di_wgtVal species=\"Species_2\">3.0</di_wgtVal>");
    oOut.write("</di_weibullGapTheta>");
    oOut.write("<di_maxGapDensity>0</di_maxGapDensity>");
    oOut.write("<di_gapFunction>");
    oOut.write("<di_gfVal species=\"Species_1\">0</di_gfVal>");
    oOut.write("<di_gfVal species=\"Species_2\">0</di_gfVal>");
    oOut.write("</di_gapFunction>");
    oOut.write("<di_canopyFunction>");
    oOut.write("<di_cfVal species=\"Species_1\">0</di_cfVal>");
    oOut.write("<di_cfVal species=\"Species_2\">0</di_cfVal>");
    oOut.write("</di_canopyFunction>");
    oOut.write("<di_seedDistributionMethod>0</di_seedDistributionMethod>");
    oOut.write("</disperse>");
    oOut.write("<establishment>");
    oOut.write("<es_gapMossCanopyFav>");
    oOut.write("<es_gmcfVal species=\"Species_1\">0.129</es_gmcfVal>");
    oOut.write("<es_gmcfVal species=\"Species_2\">0.48</es_gmcfVal>");
    oOut.write("</es_gapMossCanopyFav>");
    oOut.write("<es_organicsCanopyFav>");
    oOut.write("<es_ocfVal species=\"Species_1\">0.983</es_ocfVal>");
    oOut.write("<es_ocfVal species=\"Species_2\">0.391</es_ocfVal>");
    oOut.write("</es_organicsCanopyFav>");
    oOut.write("<es_freshLogCanopyFav>");
    oOut.write("<es_flcfVal species=\"Species_1\">0.353</es_flcfVal>");
    oOut.write("<es_flcfVal species=\"Species_2\">0.424</es_flcfVal>");
    oOut.write("</es_freshLogCanopyFav>");
    oOut.write("<es_mossyLogCanopyFav>");
    oOut.write("<es_mlcfVal species=\"Species_1\">0.058</es_mlcfVal>");
    oOut.write("<es_mlcfVal species=\"Species_2\">0.024</es_mlcfVal>");
    oOut.write("</es_mossyLogCanopyFav>");
    oOut.write("<es_canopyMossCanopyFav>");
    oOut.write("<es_cmcfVal species=\"Species_1\">0.0090</es_cmcfVal>");
    oOut.write("<es_cmcfVal species=\"Species_2\">0.01</es_cmcfVal>");
    oOut.write("</es_canopyMossCanopyFav>");
    oOut.write("<es_gapMossGapFav>");
    oOut.write("<es_gmgfVal species=\"Species_1\">0.925</es_gmgfVal>");
    oOut.write("<es_gmgfVal species=\"Species_2\">0.213</es_gmgfVal>");
    oOut.write("</es_gapMossGapFav>");
    oOut.write("<es_organicsGapFav>");
    oOut.write("<es_ogfVal species=\"Species_1\">0.913</es_ogfVal>");
    oOut.write("<es_ogfVal species=\"Species_2\">0.179</es_ogfVal>");
    oOut.write("</es_organicsGapFav>");
    oOut.write("<es_freshLogGapFav>");
    oOut.write("<es_flgfVal species=\"Species_1\">0.967</es_flgfVal>");
    oOut.write("<es_flgfVal species=\"Species_2\">0.319</es_flgfVal>");
    oOut.write("</es_freshLogGapFav>");
    oOut.write("<es_mossyLogGapFav>");
    oOut.write("<es_mlgfVal species=\"Species_1\">0.7</es_mlgfVal>");
    oOut.write("<es_mlgfVal species=\"Species_2\">0.048</es_mlgfVal>");
    oOut.write("</es_mossyLogGapFav>");
    oOut.write("<es_canopyMossGapFav>");
    oOut.write("<es_cmgfVal species=\"Species_1\">0.894</es_cmgfVal>");
    oOut.write("<es_cmgfVal species=\"Species_2\">0.0010</es_cmgfVal>");
    oOut.write("</es_canopyMossGapFav>");
    oOut.write(
        "<es_useSeedlingEfficiencyRoutine>0</es_useSeedlingEfficiencyRoutine>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file using an old-style non-gap-based substrate
   * establishment.
   * @throws IOException
   * @return String
   */
  private String WriteOldNonGapSubstrateFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>substrate</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>non-gap disperse</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>no gap substrate establishment</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<substrate>");
    oOut.write("<su_gapmossDecayAlpha>-5.1E-4</su_gapmossDecayAlpha>");
    oOut.write("<su_gapmossDecayBeta>4.4</su_gapmossDecayBeta>");
    oOut.write("<su_organicDecayAlpha>-7.0E-4</su_organicDecayAlpha>");
    oOut.write("<su_organicDecayBeta>4.3</su_organicDecayBeta>");
    oOut.write("<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>");
    oOut.write("<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>");
    oOut.write("<su_mossyLogToCanMoss>0.55</su_mossyLogToCanMoss>");
    oOut.write("<su_maxNumberDecayYears>50</su_maxNumberDecayYears>");
    oOut.write("<su_initialGapMoss>0.0</su_initialGapMoss>");
    oOut.write("<su_initialOrganic>0.0</su_initialOrganic>");
    oOut.write("<su_initialFreshLog>0.01</su_initialFreshLog>");
    oOut.write("<su_initialMossyLog>0.11</su_initialMossyLog>");
    oOut.write("<su_partialCutGapMoss>0.17</su_partialCutGapMoss>");
    oOut.write("<su_partialCutOrganic>0.15</su_partialCutOrganic>");
    oOut.write("<su_partialCutFreshLog>0.06</su_partialCutFreshLog>");
    oOut.write("<su_partialCutMossyLog>0.04</su_partialCutMossyLog>");
    oOut.write("<su_gapCutGapMoss>0.45</su_gapCutGapMoss>");
    oOut.write("<su_gapCutOrganic>0.34</su_gapCutOrganic>");
    oOut.write("<su_gapCutFreshLog>0.09</su_gapCutFreshLog>");
    oOut.write("<su_gapCutMossyLog>0.0</su_gapCutMossyLog>");
    oOut.write("<su_clearCutGapMoss>0.36</su_clearCutGapMoss>");
    oOut.write("<su_clearCutOrganic>0.34</su_clearCutOrganic>");
    oOut.write("<su_clearCutFreshLog>0.17</su_clearCutFreshLog>");
    oOut.write("<su_clearCutMossyLog>0.01</su_clearCutMossyLog>");
    oOut.write("<su_propOfDeadFall>");
    oOut.write("<su_podfVal species=\"Species_1\">0.7</su_podfVal>");
    oOut.write("<su_podfVal species=\"Species_2\">0.7</su_podfVal>");
    oOut.write("</su_propOfDeadFall>");
    oOut.write("<su_propOfFallUproot>");
    oOut.write("<su_pofuVal species=\"Species_1\">0.5</su_pofuVal>");
    oOut.write("<su_pofuVal species=\"Species_2\">0.5</su_pofuVal>");
    oOut.write("</su_propOfFallUproot>");
    oOut.write("<su_rootTipupFactor>3.1</su_rootTipupFactor>");
    oOut.write("</substrate>");
    oOut.write("<disperse>");
    oOut.write("<di_minDbhForReproduction>");
    oOut.write("<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>");
    oOut.write("<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>");
    oOut.write("</di_minDbhForReproduction>");
    oOut.write("<di_weibullCanopySTR>");
    oOut.write("<di_wcsVal species=\"Species_1\">3.8650002</di_wcsVal>");
    oOut.write("<di_wcsVal species=\"Species_2\">2.4514</di_wcsVal>");
    oOut.write("</di_weibullCanopySTR>");
    oOut.write("<di_weibullCanopyBeta>");
    oOut.write("<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>");
    oOut.write("<di_wcbVal species=\"Species_2\">2.0</di_wcbVal>");
    oOut.write("</di_weibullCanopyBeta>");
    oOut.write("<di_weibullCanopyDispersal>");
    oOut.write("<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>");
    oOut.write("<di_wcdVal species=\"Species_2\">1.82E-4</di_wcdVal>");
    oOut.write("</di_weibullCanopyDispersal>");
    oOut.write("<di_weibullCanopyTheta>");
    oOut.write("<di_wctVal species=\"Species_1\">3.0</di_wctVal>");
    oOut.write("<di_wctVal species=\"Species_2\">3.0</di_wctVal>");
    oOut.write("</di_weibullCanopyTheta>");
    oOut.write("<di_canopyFunction>");
    oOut.write("<di_cfVal species=\"Species_1\">0</di_cfVal>");
    oOut.write("<di_cfVal species=\"Species_2\">0</di_cfVal>");
    oOut.write("</di_canopyFunction>");
    oOut.write("<di_seedDistributionMethod>0</di_seedDistributionMethod>");
    oOut.write("</disperse>");
    oOut.write("<establishment>");
    oOut.write("<es_gapMossCanopyFav>");
    oOut.write("<es_gmcfVal species=\"Species_1\">0.129</es_gmcfVal>");
    oOut.write("<es_gmcfVal species=\"Species_2\">0.48</es_gmcfVal>");
    oOut.write("</es_gapMossCanopyFav>");
    oOut.write("<es_organicsCanopyFav>");
    oOut.write("<es_ocfVal species=\"Species_1\">0.983</es_ocfVal>");
    oOut.write("<es_ocfVal species=\"Species_2\">0.391</es_ocfVal>");
    oOut.write("</es_organicsCanopyFav>");
    oOut.write("<es_freshLogCanopyFav>");
    oOut.write("<es_flcfVal species=\"Species_1\">0.353</es_flcfVal>");
    oOut.write("<es_flcfVal species=\"Species_2\">0.424</es_flcfVal>");
    oOut.write("</es_freshLogCanopyFav>");
    oOut.write("<es_mossyLogCanopyFav>");
    oOut.write("<es_mlcfVal species=\"Species_1\">0.058</es_mlcfVal>");
    oOut.write("<es_mlcfVal species=\"Species_2\">0.024</es_mlcfVal>");
    oOut.write("</es_mossyLogCanopyFav>");
    oOut.write("<es_canopyMossCanopyFav>");
    oOut.write("<es_cmcfVal species=\"Species_1\">0.0090</es_cmcfVal>");
    oOut.write("<es_cmcfVal species=\"Species_2\">0.01</es_cmcfVal>");
    oOut.write("</es_canopyMossCanopyFav>");
    oOut.write(
        "<es_useSeedlingEfficiencyRoutine>0</es_useSeedlingEfficiencyRoutine>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file using an old-style non-gap-non-substrate
   * establishment.
   * @throws IOException
   * @return String
   */
  private String WriteOldNonGapNonSubstrateFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>non-gap disperse</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>no gap no substrate establishment</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<disperse>");
    oOut.write("<di_minDbhForReproduction>");
    oOut.write("<di_mdfrVal species=\"Species_1\">10.0</di_mdfrVal>");
    oOut.write("<di_mdfrVal species=\"Species_2\">10.0</di_mdfrVal>");
    oOut.write("</di_minDbhForReproduction>");
    oOut.write("<di_weibullCanopySTR>");
    oOut.write("<di_wcsVal species=\"Species_1\">0.089999996</di_wcsVal>");
    oOut.write("<di_wcsVal species=\"Species_2\">0.089999996</di_wcsVal>");
    oOut.write("</di_weibullCanopySTR>");
    oOut.write("<di_weibullCanopyBeta>");
    oOut.write("<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>");
    oOut.write("<di_wcbVal species=\"Species_2\">2.0</di_wcbVal>");
    oOut.write("</di_weibullCanopyBeta>");
    oOut.write("<di_weibullCanopyDispersal>");
    oOut.write("<di_wcdVal species=\"Species_1\">3.622E-5</di_wcdVal>");
    oOut.write("<di_wcdVal species=\"Species_2\">7.435E-5</di_wcdVal>");
    oOut.write("</di_weibullCanopyDispersal>");
    oOut.write("<di_weibullCanopyTheta>");
    oOut.write("<di_wctVal species=\"Species_1\">3.0</di_wctVal>");
    oOut.write("<di_wctVal species=\"Species_2\">3.0</di_wctVal>");
    oOut.write("</di_weibullCanopyTheta>");
    oOut.write("<di_canopyFunction>");
    oOut.write("<di_cfVal species=\"Species_1\">0</di_cfVal>");
    oOut.write("<di_cfVal species=\"Species_2\">0</di_cfVal>");
    oOut.write("</di_canopyFunction>");
    oOut.write("<di_seedDistributionMethod>0</di_seedDistributionMethod>");
    oOut.write("</disperse>");
    oOut.write("<establishment>");
    oOut.write(
        "<es_useSeedlingEfficiencyRoutine>0</es_useSeedlingEfficiencyRoutine>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid establishment parameter file with all the behaviors.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  private String WriteValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Density Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write(
        "<behaviorName>Microtopographic Substrate Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>germination</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>No Gap Substrate Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Gap Substrate Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    oOut.write("<es_densDepSlope>");
    oOut.write("<es_ddsVal species=\"Species_2\">0.2</es_ddsVal>");
    oOut.write("<es_ddsVal species=\"Species_1\">0.4</es_ddsVal>");
    oOut.write("</es_densDepSlope>");
    oOut.write("<es_densDepSteepness>");
    oOut.write("<es_ddstVal species=\"Species_2\">-0.2</es_ddstVal>");
    oOut.write("<es_ddstVal species=\"Species_1\">-0.1</es_ddstVal>");
    oOut.write("</es_densDepSteepness>");
    LightTestFiles.WriteOptimumGLI(oOut);
    LightTestFiles.WriteHighSlope(oOut);
    LightTestFiles.WriteLowSlope(oOut);
    LightTestFiles.WriteLightExtCoeffs(oOut);
    LightTestFiles.WriteLightStuff(oOut);
    SubstrateTestFiles.WriteCanopyDecayedLogFavorability(oOut);
    SubstrateTestFiles.WriteCanopyForestFloorLitterFavorability(oOut);
    SubstrateTestFiles.WriteCanopyForestFloorMossFavorability(oOut);
    SubstrateTestFiles.WriteCanopyFreshLogFavorability(oOut);
    SubstrateTestFiles.WriteCanopyScarifiedSoilFavorability(oOut);
    SubstrateTestFiles.WriteCanopyTipUpFavorability(oOut);
    SubstrateTestFiles.WriteGapDecayedLogFavorability(oOut);
    SubstrateTestFiles.WriteGapForestFloorLitterFavorability(oOut);
    SubstrateTestFiles.WriteGapForestFloorMossFavorability(oOut);
    SubstrateTestFiles.WriteGapFreshLogFavorability(oOut);
    SubstrateTestFiles.WriteGapScarifiedSoilFavorability(oOut);
    SubstrateTestFiles.WriteGapTipUpFavorability(oOut);
    oOut.write("<ge_proportionGerminating>");
    oOut.write("<ge_pgVal species=\"Species_2\">0.68</ge_pgVal>");
    oOut.write("<ge_pgVal species=\"Species_1\">0.46</ge_pgVal>");
    oOut.write("</ge_proportionGerminating>");
    oOut.write("<es_moundProportion>0.54</es_moundProportion>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid establishment parameter file with microtopographic
   * establishment.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  private String WriteValidMicroFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seedling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>light filter</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seedling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>adultstochasticmort</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>substrate</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>non spatial disperse</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Micro Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    oOut.write("<es_moundProportion>0.54</es_moundProportion>");
    oOut.write("<es_maxRespite>6</es_maxRespite>");
    oOut.write("<es_meanMoundHeight>3</es_meanMoundHeight>");
    oOut.write("<es_moundStdDev>0.5</es_moundStdDev>");
    oOut.write("<es_meanFreshLogHeight>20</es_meanFreshLogHeight>");
    oOut.write("<es_freshLogStdDev>5</es_freshLogStdDev>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a SORTIE-ND version 6.04 parameter file with substrate-dependent
   * seed survival to test backwards compatibility.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  private String WriteVersion6_04File() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>No Gap Substrate Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Gap Substrate Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    oOut.write("<es_decayedLogCanopyFav>");
    oOut.write("<es_dlcfVal species=\"Species_1\">0.058</es_dlcfVal>");
    oOut.write("<es_dlcfVal species=\"Species_2\">0.024</es_dlcfVal>");
    oOut.write("</es_decayedLogCanopyFav>");
    oOut.write("<es_forestFloorCanopyFav>");
    oOut.write("<es_ffcfVal species=\"Species_1\">0.0090</es_ffcfVal>");
    oOut.write("<es_ffcfVal species=\"Species_2\">0.01</es_ffcfVal>");
    oOut.write("</es_forestFloorCanopyFav>");
    oOut.write("<es_freshLogCanopyFav>");
    oOut.write("<es_flcfVal species=\"Species_1\">0.353</es_flcfVal>");
    oOut.write("<es_flcfVal species=\"Species_2\">0.424</es_flcfVal>");
    oOut.write("</es_freshLogCanopyFav>");
    oOut.write("<es_scarifiedSoilCanopyFav>");
    oOut.write("<es_sscfVal species=\"Species_1\">0.129</es_sscfVal>");
    oOut.write("<es_sscfVal species=\"Species_2\">0.48</es_sscfVal>");
    oOut.write("</es_scarifiedSoilCanopyFav>");
    oOut.write("<es_tipUpCanopyFav>");
    oOut.write("<es_tucfVal species=\"Species_1\">0.983</es_tucfVal>");
    oOut.write("<es_tucfVal species=\"Species_2\">0.391</es_tucfVal>");
    oOut.write("</es_tipUpCanopyFav>");
    oOut.write("<es_decayedLogGapFav>");
    oOut.write("<es_dlgfVal species=\"Species_1\">0.7</es_dlgfVal>");
    oOut.write("<es_dlgfVal species=\"Species_2\">0.048</es_dlgfVal>");
    oOut.write("</es_decayedLogGapFav>");
    oOut.write("<es_forestFloorGapFav>");
    oOut.write("<es_ffgfVal species=\"Species_1\">0.894</es_ffgfVal>");
    oOut.write("<es_ffgfVal species=\"Species_2\">0.0010</es_ffgfVal>");
    oOut.write("</es_forestFloorGapFav>");
    oOut.write("<es_freshLogGapFav>");
    oOut.write("<es_flgfVal species=\"Species_1\">0.967</es_flgfVal>");
    oOut.write("<es_flgfVal species=\"Species_2\">0.319</es_flgfVal>");
    oOut.write("</es_freshLogGapFav>");
    oOut.write("<es_scarifiedSoilGapFav>");
    oOut.write("<es_ssgfVal species=\"Species_1\">0.925</es_ssgfVal>");
    oOut.write("<es_ssgfVal species=\"Species_2\">0.213</es_ssgfVal>");
    oOut.write("</es_scarifiedSoilGapFav>");
    oOut.write("<es_tipUpGapFav>");
    oOut.write("<es_tugfVal species=\"Species_1\">0.913</es_tugfVal>");
    oOut.write("<es_tugfVal species=\"Species_2\">0.179</es_tugfVal>");
    oOut.write("</es_tipUpGapFav>");
    oOut.write("<ge_proportionGerminating>");
    oOut.write("<ge_pgVal species=\"Species_2\">0.68</ge_pgVal>");
    oOut.write("<ge_pgVal species=\"Species_1\">0.46</ge_pgVal>");
    oOut.write("</ge_proportionGerminating>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }
}

class LightTestFiles {
  /**
   * Writes a file where there is an undamaged tree light extinction
   * coefficient not between 0 and 1.
   * @throws IOException
   * @return String
   */
  public static String WriteLightErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">1.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.2</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    WriteLightStuff(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where there is a partial damaged tree light extinction
   * coefficient not between 0 and 1.
   * @throws IOException
   * @return String
   */
  public static String WriteLightErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.2</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<es_lightExtCoeffPartDmg>");
    oOut.write("<es_lecpdVal species=\"Species_1\">0.2</es_lecpdVal>");
    oOut.write("<es_lecpdVal species=\"Species_2\">-0.23</es_lecpdVal>");
    oOut.write("</es_lightExtCoeffPartDmg>");
    oOut.write("<es_lightExtCoeffFullDmg>");
    oOut.write("<es_lecfdVal species=\"Species_2\">0.8</es_lecfdVal>");
    oOut.write("<es_lecfdVal species=\"Species_1\">0.9</es_lecfdVal>");
    oOut.write("</es_lightExtCoeffFullDmg>");
    WriteLightStuff(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where there is a full damaged tree light extinction
   * coefficient not between 0 and 1.
   * @throws IOException
   * @return String
   */
  public static String WriteLightErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.2</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<es_lightExtCoeffPartDmg>");
    oOut.write("<es_lecpdVal species=\"Species_1\">0.2</es_lecpdVal>");
    oOut.write("<es_lecpdVal species=\"Species_2\">0.23</es_lecpdVal>");
    oOut.write("</es_lightExtCoeffPartDmg>");
    oOut.write("<es_lightExtCoeffFullDmg>");
    oOut.write("<es_lecfdVal species=\"Species_2\">1.8</es_lecfdVal>");
    oOut.write("<es_lecfdVal species=\"Species_1\">0.9</es_lecfdVal>");
    oOut.write("</es_lightExtCoeffFullDmg>");
    WriteLightStuff(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file where the value for light
   * height is less than 0.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>-2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light parameter file where there are values for
   * optimum GLI not between 0 and 100.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    oOut.write("<es_optimumGLI>");
    oOut.write("<es_ogVal species=\"Species_2\">23</es_ogVal>");
    oOut.write("<es_ogVal species=\"Species_1\">187</es_ogVal>");
    oOut.write("</es_optimumGLI>");
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    WriteLightStuff(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light parameter file where the value for
   * m_fBeamFractionOfGlobalRadiation is not a valid proportion.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>-0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a PR Establishment parameter file where the value for
   * m_iJulianDayGrowthStarts is not between 1 and 365 (inclusive).
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>1105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light parameter file where the value for
   * m_iJulianDayGrowthEnds is not between 1 and 365 (inclusive).
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile8() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>-258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light parameter file where the value for
   * m_iNumAltDiv is not greater than 0.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile9() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>-12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light parameter file where the value for
   * m_iNumAltDiv is not greater than 0.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile10() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteLightBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>-18</li_numAziGrids>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light establishment parameter file where there is a snag size
   * class 1 light extinction coefficient not between 0 and 1.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile11() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallightshell</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.21</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<es_lightExtCoeffPartDmg>");
    oOut.write("<es_lecpdVal species=\"Species_1\">0.22</es_lecpdVal>");
    oOut.write("<es_lecpdVal species=\"Species_2\">0.23</es_lecpdVal>");
    oOut.write("</es_lightExtCoeffPartDmg>");
    oOut.write("<es_lightExtCoeffFullDmg>");
    oOut.write("<es_lecfdVal species=\"Species_2\">0.8</es_lecfdVal>");
    oOut.write("<es_lecfdVal species=\"Species_1\">0.9</es_lecfdVal>");
    oOut.write("</es_lightExtCoeffFullDmg>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_2\">-0.7</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.6</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.5</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.4</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.1</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_1\">1</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    WriteLightStuff(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light establishment parameter file where there is a snag size
   * class 2 light extinction coefficient not between 0 and 1.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile12() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallightshell</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.21</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<es_lightExtCoeffPartDmg>");
    oOut.write("<es_lecpdVal species=\"Species_1\">0.22</es_lecpdVal>");
    oOut.write("<es_lecpdVal species=\"Species_2\">0.23</es_lecpdVal>");
    oOut.write("</es_lightExtCoeffPartDmg>");
    oOut.write("<es_lightExtCoeffFullDmg>");
    oOut.write("<es_lecfdVal species=\"Species_2\">0.8</es_lecfdVal>");
    oOut.write("<es_lecfdVal species=\"Species_1\">0.9</es_lecfdVal>");
    oOut.write("</es_lightExtCoeffFullDmg>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.7</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.6</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.5</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_1\">5.4</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.1</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_1\">1</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    WriteLightStuff(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light establishment parameter file where there is a snag size
   * class 3 light extinction coefficient not between 0 and 1.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile13() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallightshell</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallightshell</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.21</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<es_lightExtCoeffPartDmg>");
    oOut.write("<es_lecpdVal species=\"Species_1\">0.22</es_lecpdVal>");
    oOut.write("<es_lecpdVal species=\"Species_2\">0.23</es_lecpdVal>");
    oOut.write("</es_lightExtCoeffPartDmg>");
    oOut.write("<es_lightExtCoeffFullDmg>");
    oOut.write("<es_lecfdVal species=\"Species_2\">0.8</es_lecfdVal>");
    oOut.write("<es_lecfdVal species=\"Species_1\">0.9</es_lecfdVal>");
    oOut.write("</es_lightExtCoeffFullDmg>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.7</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.6</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.5</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.4</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.1</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_1\">-1</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    WriteLightStuff(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light establishment parameter file where the first snag age
   * class isn't zero.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile14() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallightshell</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_snagAgeClass1>-10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light establishment parameter file where the second snag age
   * class isn't zero.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile15() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallightshell</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>-17</li_snagAgeClass2>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a light establishment parameter file where the first snag age
   * class is greater than the second class.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteLightErrorFile16() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallightshell</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");

    oOut.write("<establishment>");
    WriteOptimumGLI(oOut);
    WriteHighSlope(oOut);
    WriteLowSlope(oOut);
    WriteLightExtCoeffs(oOut);
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_snagAgeClass1>20</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  public static void WriteValidateFilePart1(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  public static void WriteOptimumGLI(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<es_optimumGLI>");
    oOut.write("<es_ogVal species=\"Species_2\">23</es_ogVal>");
    oOut.write("<es_ogVal species=\"Species_1\">87</es_ogVal>");
    oOut.write("</es_optimumGLI>");
  }

  public static void WriteLowSlope(FileWriter oOut) throws java.io.IOException {
    oOut.write("<es_lowSlope>");
    oOut.write("<es_lsVal species=\"Species_2\">0.034</es_lsVal>");
    oOut.write("<es_lsVal species=\"Species_1\">0.0098</es_lsVal>");
    oOut.write("</es_lowSlope>");
  }

  public static void WriteHighSlope(FileWriter oOut) throws java.io.IOException {
    oOut.write("<es_highSlope>");
    oOut.write("<es_hsVal species=\"Species_2\">0.0023</es_hsVal>");
    oOut.write("<es_hsVal species=\"Species_1\">0.0098</es_hsVal>");
    oOut.write("</es_highSlope>");
  }

  public static void WriteLightStuff(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
  }

  public static void WriteLightExtCoeffs(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.21</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<es_lightExtCoeffPartDmg>");
    oOut.write("<es_lecpdVal species=\"Species_1\">0.22</es_lecpdVal>");
    oOut.write("<es_lecpdVal species=\"Species_2\">0.23</es_lecpdVal>");
    oOut.write("</es_lightExtCoeffPartDmg>");
    oOut.write("<es_lightExtCoeffFullDmg>");
    oOut.write("<es_lecfdVal species=\"Species_2\">0.8</es_lecfdVal>");
    oOut.write("<es_lecfdVal species=\"Species_1\">0.9</es_lecfdVal>");
    oOut.write("</es_lightExtCoeffFullDmg>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.7</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.6</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.5</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.4</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.1</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_1\">1</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
  }

  public static void WriteLightBehaviorList(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }
}

/**
 * Writes substrate testing files.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
class SubstrateTestFiles {

  /**
   * Canopy decayed log favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    oOut.write("<es_decayedLogCanopyFav>");
    oOut.write("<es_dlcfVal species=\"Species_1\">-0.058</es_dlcfVal>");
    oOut.write("<es_dlcfVal species=\"Species_2\">0.024</es_dlcfVal>");
    oOut.write("</es_decayedLogCanopyFav>");
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Canopy forest floor litter favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    oOut.write("<es_forestFloorLitterCanopyFav>");
    oOut.write("<es_fflcfVal species=\"Species_1\">1.0090</es_fflcfVal>");
    oOut.write("<es_fflcfVal species=\"Species_2\">0.01</es_fflcfVal>");
    oOut.write("</es_forestFloorLitterCanopyFav>");
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Canopy fresh log favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    oOut.write("<es_freshLogCanopyFav>");
    oOut.write("<es_flcfVal species=\"Species_1\">-0.353</es_flcfVal>");
    oOut.write("<es_flcfVal species=\"Species_2\">0.424</es_flcfVal>");
    oOut.write("</es_freshLogCanopyFav>");
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Canopy scarified soil favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    oOut.write("<es_scarifiedSoilCanopyFav>");
    oOut.write("<es_sscfVal species=\"Species_1\">0.129</es_sscfVal>");
    oOut.write("<es_sscfVal species=\"Species_2\">-0.48</es_sscfVal>");
    oOut.write("</es_scarifiedSoilCanopyFav>");
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Canopy tip up favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    oOut.write("<es_tipUpCanopyFav>");
    oOut.write("<es_tucfVal species=\"Species_1\">0.983</es_tucfVal>");
    oOut.write("<es_tucfVal species=\"Species_2\">-0.391</es_tucfVal>");
    oOut.write("</es_tipUpCanopyFav>");
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Gap decayed log favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    oOut.write("<es_decayedLogGapFav>");
    oOut.write("<es_dlgfVal species=\"Species_1\">-0.7</es_dlgfVal>");
    oOut.write("<es_dlgfVal species=\"Species_2\">0.048</es_dlgfVal>");
    oOut.write("</es_decayedLogGapFav>");
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Gap forest floor favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    oOut.write("<es_forestFloorLitterGapFav>");
    oOut.write("<es_fflgfVal species=\"Species_1\">0.894</es_fflgfVal>");
    oOut.write("<es_fflgfVal species=\"Species_2\">-0.0010</es_fflgfVal>");
    oOut.write("</es_forestFloorLitterGapFav>");
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Gap fresh log favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile8() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    oOut.write("<es_freshLogGapFav>");
    oOut.write("<es_flgfVal species=\"Species_1\">-0.967</es_flgfVal>");
    oOut.write("<es_flgfVal species=\"Species_2\">0.319</es_flgfVal>");
    oOut.write("</es_freshLogGapFav>");
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Gap scarified soil favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile9() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    oOut.write("<es_scarifiedSoilGapFav>");
    oOut.write("<es_ssgfVal species=\"Species_1\">0.925</es_ssgfVal>");
    oOut.write("<es_ssgfVal species=\"Species_2\">-0.213</es_ssgfVal>");
    oOut.write("</es_scarifiedSoilGapFav>");
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Gap tip-up favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile10() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    oOut.write("<es_tipUpGapFav>");
    oOut.write("<es_tugfVal species=\"Species_1\">1.913</es_tugfVal>");
    oOut.write("<es_tugfVal species=\"Species_2\">0.179</es_tugfVal>");
    oOut.write("</es_tipUpGapFav>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Canopy forest floor moss favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile11() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    oOut.write("<es_forestFloorMossCanopyFav>");
    oOut.write("<es_ffmcfVal species=\"Species_1\">0.162</es_ffmcfVal>");
    oOut.write("<es_ffmcfVal species=\"Species_2\">-0.885</es_ffmcfVal>");
    oOut.write("</es_forestFloorMossCanopyFav>");
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    WriteGapForestFloorMossFavorability(oOut);
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Gap forest floor moss favorability isn't a proportion.
   * @throws IOException
   * @return String
   */
  public static String WriteSubstrateErrorFile12() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    LightTestFiles.WriteValidateFilePart1(oOut);

    WriteSubstrateBehaviorList(oOut);
    oOut.write("<establishment>");
    WriteCanopyDecayedLogFavorability(oOut);
    WriteCanopyForestFloorLitterFavorability(oOut);
    WriteCanopyForestFloorMossFavorability(oOut);
    WriteCanopyFreshLogFavorability(oOut);
    WriteCanopyScarifiedSoilFavorability(oOut);
    WriteCanopyTipUpFavorability(oOut);
    WriteGapDecayedLogFavorability(oOut);
    WriteGapForestFloorLitterFavorability(oOut);
    oOut.write("<es_forestFloorMossGapFav>");
    oOut.write("<es_ffmgfVal species=\"Species_1\">1.568</es_ffmgfVal>");
    oOut.write("<es_ffmgfVal species=\"Species_2\">0.911</es_ffmgfVal>");
    oOut.write("</es_forestFloorMossGapFav>");
    WriteGapFreshLogFavorability(oOut);
    WriteGapScarifiedSoilFavorability(oOut);
    WriteGapTipUpFavorability(oOut);
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  public static void WriteSubstrateBehaviorList(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Substrate Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  public static void WriteCanopyScarifiedSoilFavorability(FileWriter oOut) throws
      java.io.IOException {
    oOut.write("<es_scarifiedSoilCanopyFav>");
    oOut.write("<es_sscfVal species=\"Species_1\">0.129</es_sscfVal>");
    oOut.write("<es_sscfVal species=\"Species_2\">0.48</es_sscfVal>");
    oOut.write("</es_scarifiedSoilCanopyFav>");
  }

  public static void WriteCanopyTipUpFavorability(FileWriter oOut) throws java.
      io.
      IOException {
    oOut.write("<es_tipUpCanopyFav>");
    oOut.write("<es_tucfVal species=\"Species_1\">0.983</es_tucfVal>");
    oOut.write("<es_tucfVal species=\"Species_2\">0.391</es_tucfVal>");
    oOut.write("</es_tipUpCanopyFav>");
  }

  public static void WriteCanopyFreshLogFavorability(FileWriter oOut) throws
      java.io.
      IOException {
    oOut.write("<es_freshLogCanopyFav>");
    oOut.write("<es_flcfVal species=\"Species_1\">0.353</es_flcfVal>");
    oOut.write("<es_flcfVal species=\"Species_2\">0.424</es_flcfVal>");
    oOut.write("</es_freshLogCanopyFav>");
  }

  public static void WriteCanopyDecayedLogFavorability(FileWriter oOut) throws
      java.
      io.IOException {
    oOut.write("<es_decayedLogCanopyFav>");
    oOut.write("<es_dlcfVal species=\"Species_1\">0.058</es_dlcfVal>");
    oOut.write("<es_dlcfVal species=\"Species_2\">0.024</es_dlcfVal>");
    oOut.write("</es_decayedLogCanopyFav>");
  }

  public static void WriteCanopyForestFloorLitterFavorability(FileWriter oOut) throws
      java.
      io.IOException {
    oOut.write("<es_forestFloorLitterCanopyFav>");
    oOut.write("<es_fflcfVal species=\"Species_1\">0.0090</es_fflcfVal>");
    oOut.write("<es_fflcfVal species=\"Species_2\">0.01</es_fflcfVal>");
    oOut.write("</es_forestFloorLitterCanopyFav>");
  }

  public static void WriteCanopyForestFloorMossFavorability(FileWriter oOut) throws
      java.
      io.IOException {
    oOut.write("<es_forestFloorMossCanopyFav>");
    oOut.write("<es_ffmcfVal species=\"Species_1\">0.162</es_ffmcfVal>");
    oOut.write("<es_ffmcfVal species=\"Species_2\">0.885</es_ffmcfVal>");
    oOut.write("</es_forestFloorMossCanopyFav>");
  }

  public static void WriteGapScarifiedSoilFavorability(FileWriter oOut) throws
      java.
      io.IOException {
    oOut.write("<es_scarifiedSoilGapFav>");
    oOut.write("<es_ssgfVal species=\"Species_1\">0.925</es_ssgfVal>");
    oOut.write("<es_ssgfVal species=\"Species_2\">0.213</es_ssgfVal>");
    oOut.write("</es_scarifiedSoilGapFav>");
  }

  public static void WriteGapTipUpFavorability(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<es_tipUpGapFav>");
    oOut.write("<es_tugfVal species=\"Species_1\">0.913</es_tugfVal>");
    oOut.write("<es_tugfVal species=\"Species_2\">0.179</es_tugfVal>");
    oOut.write("</es_tipUpGapFav>");
  }

  public static void WriteGapFreshLogFavorability(FileWriter oOut) throws java.
      io.
      IOException {
    oOut.write("<es_freshLogGapFav>");
    oOut.write("<es_flgfVal species=\"Species_1\">0.967</es_flgfVal>");
    oOut.write("<es_flgfVal species=\"Species_2\">0.319</es_flgfVal>");
    oOut.write("</es_freshLogGapFav>");
  }

  public static void WriteGapDecayedLogFavorability(FileWriter oOut) throws
      java.io.
      IOException {
    oOut.write("<es_decayedLogGapFav>");
    oOut.write("<es_dlgfVal species=\"Species_1\">0.7</es_dlgfVal>");
    oOut.write("<es_dlgfVal species=\"Species_2\">0.048</es_dlgfVal>");
    oOut.write("</es_decayedLogGapFav>");
  }

  public static void WriteGapForestFloorLitterFavorability(FileWriter oOut) throws
      java.io.
      IOException {
    oOut.write("<es_forestFloorLitterGapFav>");
    oOut.write("<es_fflgfVal species=\"Species_1\">0.894</es_fflgfVal>");
    oOut.write("<es_fflgfVal species=\"Species_2\">0.0010</es_fflgfVal>");
    oOut.write("</es_forestFloorLitterGapFav>");
  }

  public static void WriteGapForestFloorMossFavorability(FileWriter oOut) throws
      java.io.
      IOException {
    oOut.write("<es_forestFloorMossGapFav>");
    oOut.write("<es_ffmgfVal species=\"Species_1\">0.568</es_ffmgfVal>");
    oOut.write("<es_ffmgfVal species=\"Species_2\">0.911</es_ffmgfVal>");
    oOut.write("</es_forestFloorMossGapFav>");
  }
}
