package javawrapper;

import junit.framework.*;
import java.util.Vector;
import java.io.FileWriter;
import java.io.File;

/**
 * Tests the Storms behavior of the DisturbanceBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestStormBehaviors
    extends TestCase {
  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  public void testFormatDataForDisplay() {
    try {
      TreePopulation oPop = new TreePopulation(null);
      oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});

      DisturbanceBehaviors oDisturbance = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no disturbance behaviors are enabled.
      oDisturbance = new DisturbanceBehaviors(null);
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  only storms is enabled.
      oDisturbance = null;
      oDisturbance = new DisturbanceBehaviors(null);
      oDisturbance.DoSetup(oPop);
      oBeh = oDisturbance.GetBehaviorByKey("Storms");
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Return Interval for Severity Storm Class 0 - 0.1",
          "Return Interval for Severity Storm Class 0.1 - 0.2",
          "Return Interval for Severity Storm Class 0.2 - 0.3",
          "Return Interval for Severity Storm Class 0.3 - 0.4",
          "Return Interval for Severity Storm Class 0.4 - 0.5",
          "Return Interval for Severity Storm Class 0.5 - 0.6",
          "Return Interval for Severity Storm Class 0.6 - 0.7",
          "Return Interval for Severity Storm Class 0.7 - 0.8",
          "Return Interval for Severity Storm Class 0.8 - 0.9",
          "Return Interval for Severity Storm Class 0.9 - 1.0",
          "Standard Deviation (lognormal or normal)",
          "Clumping Parameter (neg. binomial)",
          "Plot Storm Susceptibility Pattern",
          "Storm Damage Application",
          "Stochastic Pattern Damage Distribution"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      //Case: only storm damage applier is enabled.
      oDisturbance = null;
      oDisturbance = new DisturbanceBehaviors(null);
      oDisturbance.DoSetup(oPop);
      oBeh = oDisturbance.GetBehaviorByKey("Storm Damage Applier");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Minimum DBH for Storm Damage, in cm",
          "Storm Damage Intercept (a) for Medium Damage",
          "Storm Damage Intercept (a) for Heavy Damage",
          "Storm Intensity Coefficient (b)",
          "Storm DBH Coefficient (d)",
          "Storm - Proportion of Trees with Medium Damage that Die",
          "Storm - Proportion of Trees with Heavy Damage that Die",
          "Storm - Prop. Heavy Damage Dead Trees that Tip Up",
          "Number of Years Storm-Damaged Snags Last",
          "Number of Years Damaged Trees Take to Heal"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  Only Harvest is enabled.
      oDisturbance = null;
      oDisturbance = new DisturbanceBehaviors(null);
      oDisturbance.DoSetup(oPop);
      oBeh = oDisturbance.GetBehaviorByKey("Harvest");
      oBeh.m_bIsEnabled = true;
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  Only Natural DIsturbance is enabled.
      oDisturbance = null;
      oDisturbance = new DisturbanceBehaviors(null);
      oDisturbance.DoSetup(oPop);
      oBeh = oDisturbance.GetBehaviorByKey("Episodic Mortality");
      oBeh.m_bIsEnabled = true;
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  only selection harvest is enabled.
      oDisturbance = null;
      oDisturbance = new DisturbanceBehaviors(null);
      oDisturbance.DoSetup(oPop);
      oBeh = oDisturbance.GetBehaviorByKey("Selection Harvest");
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Selection Harvest Cut Range 1 Lower DBH (cm)",
          "Selection Harvest Cut Range 1 Upper DBH (cm)",
          "Selection Harvest Cut Range 1 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 2 Lower DBH (cm)",
          "Selection Harvest Cut Range 2 Upper DBH (cm)",
          "Selection Harvest Cut Range 2 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 3 Lower DBH (cm)",
          "Selection Harvest Cut Range 3 Upper DBH (cm)",
          "Selection Harvest Cut Range 3 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 4 Lower DBH (cm)",
          "Selection Harvest Cut Range 4 Upper DBH (cm)",
          "Selection Harvest Cut Range 4 Target Basal Area (m2/ha)",
          "Selection Harvest Initial Age"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  all are enabled.
      oDisturbance = null;
      oDisturbance = new DisturbanceBehaviors(null);
      oDisturbance.DoSetup(oPop);
      int i;
      for (i = 0; i < oDisturbance.mp_oChildBehaviors.length; i++) {
        oDisturbance.mp_oChildBehaviors[i].m_bIsEnabled = true;
        if (oDisturbance.mp_oChildBehaviors[i].GetKey().equals(
            "Storm Damage Applier")) {
          oDisturbance.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(0, 3, oPop));
        }
      }
      p_sExpected = new String[] {
          "Return Interval for Severity Storm Class 0 - 0.1",
          "Return Interval for Severity Storm Class 0.1 - 0.2",
          "Return Interval for Severity Storm Class 0.2 - 0.3",
          "Return Interval for Severity Storm Class 0.3 - 0.4",
          "Return Interval for Severity Storm Class 0.4 - 0.5",
          "Return Interval for Severity Storm Class 0.5 - 0.6",
          "Return Interval for Severity Storm Class 0.6 - 0.7",
          "Return Interval for Severity Storm Class 0.7 - 0.8",
          "Return Interval for Severity Storm Class 0.8 - 0.9",
          "Return Interval for Severity Storm Class 0.9 - 1.0",
          "Standard Deviation (lognormal or normal)",
          "Clumping Parameter (neg. binomial)",
          "Plot Storm Susceptibility Pattern",
          "Storm Damage Application",
          "Stochastic Pattern Damage Distribution",
          "Minimum DBH for Storm Damage, in cm",
          "Storm Damage Intercept (a) for Medium Damage",
          "Storm Damage Intercept (a) for Heavy Damage",
          "Storm Intensity Coefficient (b)",
          "Storm DBH Coefficient (d)",
          "Storm - Proportion of Trees with Medium Damage that Die",
          "Storm - Proportion of Trees with Heavy Damage that Die",
          "Storm - Prop. Heavy Damage Dead Trees that Tip Up",
          "Number of Years Storm-Damaged Snags Last",
          "Number of Years Damaged Trees Take to Heal",
          "Selection Harvest Cut Range 1 Lower DBH (cm)",
          "Selection Harvest Cut Range 1 Upper DBH (cm)",
          "Selection Harvest Cut Range 1 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 2 Lower DBH (cm)",
          "Selection Harvest Cut Range 2 Upper DBH (cm)",
          "Selection Harvest Cut Range 2 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 3 Lower DBH (cm)",
          "Selection Harvest Cut Range 3 Upper DBH (cm)",
          "Selection Harvest Cut Range 3 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 4 Lower DBH (cm)",
          "Selection Harvest Cut Range 4 Upper DBH (cm)",
          "Selection Harvest Cut Range 4 Target Basal Area (m2/ha)",
          "Selection Harvest Initial Age"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for disturbance.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.sMessage);
    }
  }

  /**
   * Tests parameter file reading.  This makes sure that all values for
   * storms make it where they are supposed to.
   * @throws ModelException if a test fails or a parameter file cannot be
   * written or parsed.
   */
  public void testReadParFile() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //File 1 - deterministic uniform damage pattern
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      assertEquals(oDist.m_fClass1RtrnInt.GetValue(), 0.1, 0.001);
      assertEquals(oDist.m_fClass2RtrnInt.GetValue(), 0.2, 0.001);
      assertEquals(oDist.m_fClass3RtrnInt.GetValue(), 0.3, 0.001);
      assertEquals(oDist.m_fClass4RtrnInt.GetValue(), 0.4, 0.001);
      assertEquals(oDist.m_fClass5RtrnInt.GetValue(), 0.5, 0.001);
      assertEquals(oDist.m_fClass6RtrnInt.GetValue(), 0.6, 0.001);
      assertEquals(oDist.m_fClass7RtrnInt.GetValue(), 0.7, 0.001);
      assertEquals(oDist.m_fClass8RtrnInt.GetValue(), 0.8, 0.001);
      assertEquals(oDist.m_fClass9RtrnInt.GetValue(), 0.9, 0.001);
      assertEquals(oDist.m_fClass10RtrnInt.GetValue(), 1.0, 0.001);
      assertTrue(oDist.m_iStochasticity.GetStringValue().equalsIgnoreCase(
          "deterministic"));
      assertTrue(oDist.m_iSusceptibility.GetStringValue().equalsIgnoreCase(
          "uniform"));

      assertEquals( ( (Float) oDist.mp_fMinDBH.GetValue().elementAt(0)).
                   floatValue(), 5, 0.001);
      assertEquals( ( (Float) oDist.mp_fMinDBH.GetValue().elementAt(1)).
                   floatValue(), 6, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptMed.GetValue().elementAt(
          0)).
                   floatValue(), 2.1789601, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptMed.GetValue().elementAt(
          1)).
                   floatValue(), 2.772772, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptFull.GetValue().
                     elementAt(0)).
                   floatValue(), 2.6334169, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptFull.GetValue().
                     elementAt(1)).
                   floatValue(), 4.9189817, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmIntensityCoeff.GetValue().elementAt(
          0)).
                   floatValue(), -0.2545352, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmIntensityCoeff.GetValue().elementAt(
          1)).
                   floatValue(), -1.936994, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDBHCoeff.GetValue().elementAt(0)).
                   floatValue(), 0.8280319, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDBHCoeff.GetValue().elementAt(1)).
                   floatValue(), 0.2411269, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmPropMedDmgDie.GetValue().elementAt(0)).
                   floatValue(), 0.1, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmPropMedDmgDie.GetValue().elementAt(1)).
                   floatValue(), 0.2, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmPropFullDmgDie.GetValue().elementAt(
          0)).
                   floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmPropFullDmgDie.GetValue().elementAt(
          1)).
                   floatValue(), 0.4, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmPropFullTipUp.GetValue().elementAt(0)).
                   floatValue(), 0.5, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmPropFullTipUp.GetValue().elementAt(1)).
                   floatValue(), 0.6, 0.001);

      assertEquals(4, oDist.m_iNumYearsToHeal.GetValue());
      assertEquals(3, oDist.m_iNumSnagYears.GetValue());

      new File(sFileName).delete();

      //File 2 - stochastic uniform damage pattern - skip the stuff we already
      //read correctly
      sFileName = WriteXMLValidFile2();
      oManager.InputXMLParameterFile(sFileName);
      oDist = oManager.GetDisturbanceBehaviors();
      assertTrue(oDist.m_iStochasticity.GetStringValue().equalsIgnoreCase(
          "stochastic"));
      assertTrue(oDist.m_iSusceptibility.GetStringValue().equalsIgnoreCase(
          "uniform"));
      assertTrue(oDist.m_iProbDistFunc.GetStringValue().equalsIgnoreCase(
          "lognormal"));
      assertEquals(oDist.m_fStdDev.GetValue(), 0.3, 0.001);
      new File(sFileName).delete();

      //File 3 - stochastic uniform damage pattern - skip the stuff we already
      //read correctly
      sFileName = WriteXMLValidFile3();
      oManager.InputXMLParameterFile(sFileName);
      oDist = oManager.GetDisturbanceBehaviors();
      assertTrue(oDist.m_iStochasticity.GetStringValue().equalsIgnoreCase(
          "stochastic"));
      assertTrue(oDist.m_iSusceptibility.GetStringValue().equalsIgnoreCase(
          "uniform"));
      assertTrue(oDist.m_iProbDistFunc.GetStringValue().equalsIgnoreCase(
          "negative binomial"));
      assertEquals(oDist.m_fClump.GetValue(), 0.3, 0.001);
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Storm validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests ValidateData().  Four valid files and 10 invalid files are
   * put through.
   * @throws ModelException if the validation happens incorrectly.
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 2
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 3
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 4
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 5
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Disturbance validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 4
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 4.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 5
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 6
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile6();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 6.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 7
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile7();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 7.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 8
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile8();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 8.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 9
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile9();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 9.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 10
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile10();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 10.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    System.out.println("Storm ValidateData test was successful.");

  }

  /**
   * Writes a valid file for storms with deterministic uniform damage.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  protected String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    WriteReturnIntervals(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for storms with stochastic uniform damage.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  protected String WriteXMLValidFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    WriteReturnIntervals(oOut);
    oOut.write("<st_stochasticity>1</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("<st_probFunction>0</st_probFunction>");
    oOut.write("<st_standardDeviation>0.3</st_standardDeviation>");
    oOut.write("</storm>");
    oOut.write("</paramFile>"); ;
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for storms with stochastic uniform damage and a
   * negative binomial function.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  protected String WriteXMLValidFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    WriteReturnIntervals(oOut);
    oOut.write("<st_stochasticity>1</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("<st_probFunction>2</st_probFunction>");
    oOut.write("<st_clumpingParameter>0.3</st_clumpingParameter>");
    oOut.write("</storm>");
    oOut.write("</paramFile>"); ;
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file for a uniform mapped damage pattern.  There is a
   * grid map for "Storm Susceptibility".
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLValidFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    //Generate the storm susceptibility map
    oOut.write("<grid gridName=\"Storm Susceptibility\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>5</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>3</ma_lengthYCells>");

    //Auto-generate grid values
    float fCount = 0;
    int iNumX = 20, iNumY = 44, iX, iY;
    for (iX = 0; iX < iNumX; iX++) {
      for (iY = 0; iY < iNumY; iY++) {
        String sVal = "<ma_v x=\"" + iX + "\" y=\"" + iY + "\"><fl c=\"0\">" +
            fCount + "</fl></ma_v>";
        oOut.write(sVal);
        fCount += 0.001;
      }
    }
    oOut.write("</grid>");
    WriteBehaviorList(oOut);
    WriteReturnIntervals(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file for a stochastic mapped damage pattern.  There is a
   * grid map for "Storm Susceptibility".
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLValidFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    //Generate the storm susceptibility map
    oOut.write("<grid gridName=\"Storm Susceptibility\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>5</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>3</ma_lengthYCells>");

    //Auto-generate grid values
    float fCount = 0;
    int iNumX = 20, iNumY = 44, iX, iY;
    for (iX = 0; iX < iNumX; iX++) {
      for (iY = 0; iY < iNumY; iY++) {
        String sVal = "<ma_v x=\"" + iX + "\" y=\"" + iY + "\"><fl c=\"0\">" +
            fCount + "</fl></ma_v>";
        oOut.write(sVal);
        fCount += 0.001;
      }
    }
    oOut.write("</grid>");
    WriteBehaviorList(oOut);
    WriteReturnIntervals(oOut);
    oOut.write("<st_stochasticity>1</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("<st_probFunction>1</st_probFunction>");
    oOut.write("<st_standardDeviation>0.3</st_standardDeviation>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file where there are grid maps for both "Storm Damage"
   * and "Storm Susceptibility" and the grid cell resolutions don't match.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    //Generate the storm susceptibility map
    oOut.write("<grid gridName=\"Storm Susceptibility\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>5</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>3</ma_lengthYCells>");

    //Auto-generate grid values
    int iNumX = 20, iNumY = 44, iX, iY;
    for (iX = 0; iX < iNumX; iX++) {
      for (iY = 0; iY < iNumY; iY++) {
        String sVal = "<ma_v x=\"" + iX + "\" y=\"" + iY + "\"><fl c=\"0\">" +
            0.5 + "</fl></ma_v>";
        oOut.write(sVal);
      }
    }
    oOut.write("</grid>");
    oOut.write("<grid gridName=\"Storm Damage\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"dmg_index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>10</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>10</ma_lengthYCells>");
    oOut.write("</grid>");
    WriteBehaviorList(oOut);
    WriteReturnIntervals(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>"); ;
    oOut.close();
    return sFileName;

  }

  /**
   * Writes a parameter file where the damage pattern is set to "mapped" but
   * there is no "Storm Susceptibility" grid map in the parameter file.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    WriteReturnIntervals(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file where the damage pattern is set to "mapped", there
   * is a grid map for "Storm Susceptibility" in the parameter file, but not all
   * values are between 0 and 1.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    //Generate the storm susceptibility map
    oOut.write("<grid gridName=\"Storm Susceptibility\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>5</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>3</ma_lengthYCells>");

    //Auto-generate grid values
    int iNumX = 20, iNumY = 44, iX, iY;
    for (iX = 0; iX < iNumX; iX++) {
      for (iY = 0; iY < iNumY; iY++) {
        String sVal = "<ma_v x=\"" + iX + "\" y=\"" + iY + "\"><fl c=\"0\">" +
            1.5 + "</fl></ma_v>";
        oOut.write(sVal);
      }
    }
    oOut.write("</grid>");
    WriteBehaviorList(oOut);
    WriteReturnIntervals(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>"); ;
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file where not all storm return interval values are
   * positive.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>-1.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file where the storm damage applier behavior is enabled
   * but not the storm behavior.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm damage applier</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<storm>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.1</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">0.2</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">0.3</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0.4</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the number of years to heal is negative.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 29, 2004: Created (LEM)
   */
  protected String WriteXMLErrorFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.1</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">0.2</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">0.3</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0.4</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>-4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the proportion of trees with medium damage to die is
   * not a proportion.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 4, 2005: Created (LEM)
   */
  protected String WriteXMLErrorFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.8280319</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">1.2411269</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">0.8280319</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0.2411269</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the proportion of trees with total damage to die is
   * not a proportion.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 4, 2005: Created (LEM)
   */
  protected String WriteXMLErrorFile8() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.8280319</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">1.2411269</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">-0.8280319</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0.2411269</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the proportion of trees with total damage that tip up
   * is not a proportion.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>May 2, 2005: Created (LEM)
   */
  protected String WriteXMLErrorFile9() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.8280319</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">1.2411269</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">0.8280319</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0.2411269</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">1.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the snag lifetime is negative.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>May 2, 2005: Created (LEM)
   */
  protected String WriteXMLErrorFile10() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.8280319</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">1.2411269</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">0.8280319</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0.2411269</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>-3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Tests a table set against a list of expected members in the table.  It
   * does not care how many tables are defined in the vector.  It will
   * throw a fail if there are members missing or members that are not
   * expected.  Species are not checked.  Blanks are skipped.
   * @param p_oTablesToTest Vector of tables to test.
   * @param p_sExpected Expected member list.
   */
  private void TestTable(Vector p_oTablesToTest, String[] p_sExpected) {
    Object[][] p_oData;
    Vector p_oOneTable;
    String sMemberName;
    int[] p_iFoundTimes = new int[p_sExpected.length]; //holds the number of
    //times each member was found
    int iNumberOfTables = p_oTablesToTest.size(),
        i, j, k;
    boolean bFound;

    //Initialize the number of times found array to all 0s
    for (i = 0; i < p_iFoundTimes.length; i++) {
      p_iFoundTimes[i] = 0;
    }

    //Analyze each table
    for (i = 0; i < iNumberOfTables; i++) {
      p_oOneTable = (Vector) p_oTablesToTest.elementAt(i);

      //The second element in the vector is the data
      p_oData = (Object[][]) p_oOneTable.elementAt(1);

      //Go through each table row
      for (j = 0; j < p_oData.length; j++) {

        sMemberName = (String) p_oData[j][0];
        bFound = false;

        if (!sMemberName.equals(" ")) {

          //Does it match a member of the expected list?
          for (k = 0; k < p_sExpected.length; k++) {

            if (sMemberName.equals(p_sExpected[k])) {

              bFound = true;
              p_iFoundTimes[k]++;

            }
          }
        }

        //Did we find our member in the expected list?
        if (!bFound) {
          fail(sMemberName + " unexpectedly found in table.");
        }
      }
    }

    //Now look at the number of times we found each expected member - if
    //not 1 for each, fail
    for (i = 0; i < p_iFoundTimes.length; i++) {
      assertEquals("Found " + p_sExpected[i] + " unexpected number of times.",
                   1, p_iFoundTimes[i]);
    }
  }

  private void WriteFileStart(FileWriter oOut) throws java.io.IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  private void WriteBehaviorList(FileWriter oOut) throws java.io.IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm damage applier</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  private void WriteReturnIntervals(FileWriter oOut) throws java.io.IOException {
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.1</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">0.2</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">0.3</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0.4</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
  }
}
